package com.example.domticacongestosfaciales

import android.Manifest
import android.app.AlertDialog
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothDevice
import android.bluetooth.BluetoothManager
import android.bluetooth.BluetoothSocket
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.content.res.Configuration
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.text.Editable
import android.text.TextWatcher
import android.util.Log
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ArrayAdapter
import android.widget.Button
import android.widget.EditText
import android.widget.Spinner
import android.widget.TextView
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import com.example.domticacongestosfaciales.databinding.FragmentBTConfigBinding
import com.example.domticacongestosfaciales.databinding.FragmentMQTTConfigBinding
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import java.io.IOException
import java.util.UUID




const val REQUEST_ENABLE_BT = 1
class BTConfigFragment : Fragment() {

    private lateinit var binding:FragmentBTConfigBinding

    //BluetoothAdapter
    lateinit var mBtAdapter: BluetoothAdapter
    var mAddressDevices: ArrayAdapter<String>? = null
    var mNameDevices: ArrayAdapter<String>? = null

    companion object {
        var m_myUUID: UUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB")
        var m_bluetoothSocket: BluetoothSocket? = null

        var m_isConnected: Boolean = false
        var m_address: String = ""
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        binding = FragmentBTConfigBinding.inflate(inflater,container,false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        //permisos()

        mAddressDevices = ArrayAdapter(requireContext(), android.R.layout.simple_list_item_1)
        mNameDevices = ArrayAdapter(requireContext(), android.R.layout.simple_list_item_1)

        //Para poder guardar la dirección del dispositivo y usarlo en InitialFragment para conectarse directamente
        val BTpreferences = requireContext().getSharedPreferences("BTPreferences", Context.MODE_PRIVATE)


        val idBtnOnBT = binding.idBtnOnBT
        val idBtnOffBT = binding.idBtnOffBT
        val idBtnConect = binding.idBtnConect

        val idBtnDispBT = binding.idBtnDispBT
        val idSpinDisp = binding.idSpinDisp
        //--------------------------------------------------
        //--------------------------------------------------
        val someActivityResultLauncher = registerForActivityResult(
            ActivityResultContracts.StartActivityForResult()
        ) { result ->
            if (result.resultCode == REQUEST_ENABLE_BT) {
                Log.i("MainActivity", "ACTIVIDAD REGISTRADA")
            }
        }

        //Inicializacion del bluetooth adapter
        mBtAdapter = (requireContext().getSystemService(Context.BLUETOOTH_SERVICE) as BluetoothManager).adapter

        //Checar si esta encendido o apagado
        if (mBtAdapter == null) {
            Toast.makeText(requireContext(), "Bluetooth no está disponible en este dipositivo", Toast.LENGTH_LONG).show()
        } else {
            Toast.makeText(requireContext(), "Bluetooth está disponible en este dispositivo", Toast.LENGTH_LONG).show()
        }
        //--------------------------------------------------
        //--------------------------------------------------
        //Boton Encender bluetooth
        idBtnOnBT.setOnClickListener {
            if (mBtAdapter.isEnabled) {
                //Si ya está activado
                Toast.makeText(requireContext(), "Bluetooth ya se encuentra activado", Toast.LENGTH_LONG).show()
            } else {
                //Encender Bluetooth
                val enableBtIntent = Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE)
                if (ActivityCompat.checkSelfPermission(
                        requireContext(),
                        Manifest.permission.BLUETOOTH_CONNECT
                    ) != PackageManager.PERMISSION_GRANTED
                ) {
                    Log.i("MainActivity", "ActivityCompat#requestPermissions")
                }
                someActivityResultLauncher.launch(enableBtIntent)
            }
        }

        //Boton apagar bluetooth
        idBtnOffBT.setOnClickListener {
            if (!mBtAdapter.isEnabled) {
                //Si ya está desactivado
                Toast.makeText(requireContext(), "Bluetooth ya se encuentra desactivado", Toast.LENGTH_LONG).show()
            } else {
                //Apagar Bluetooth
                //mBtAdapter.disable() //Está deprecated, ya no va, así que cree una función aparte
                disconnect()
                Toast.makeText(requireContext(), "Se ha desactivado el bluetooth", Toast.LENGTH_LONG).show()
            }
        }

        //Boton dispositivos emparejados
        idBtnDispBT.setOnClickListener {
            if (mBtAdapter.isEnabled) {

                val pairedDevices: Set<BluetoothDevice>? = mBtAdapter?.bondedDevices
                mAddressDevices!!.clear()
                mNameDevices!!.clear()

                pairedDevices?.forEach { device ->
                    val deviceName = device.name
                    val deviceHardwareAddress = device.address // MAC address
                    mAddressDevices!!.add(deviceHardwareAddress)
                    //........... EN ESTE PUNTO GUARDO LOS NOMBRE A MOSTRARSE EN EL COMBO BOX
                    mNameDevices!!.add(deviceName)
                }

                //ACTUALIZO LOS DISPOSITIVOS
                idSpinDisp.setAdapter(mNameDevices)
            } else {
                val noDevices = "Ningun dispositivo pudo ser emparejado"
                mAddressDevices!!.add(noDevices)
                mNameDevices!!.add(noDevices)
                Toast.makeText(requireContext(), "Primero vincule un dispositivo bluetooth", Toast.LENGTH_LONG).show()
            }
        }

        idBtnConect.setOnClickListener {
            try {
                if ((m_bluetoothSocket == null || !m_isConnected) && idSpinDisp.selectedItemPosition!=-1) {
                    val IntValSpin = idSpinDisp.selectedItemPosition
                    m_address = mAddressDevices!!.getItem(IntValSpin).toString()
                    Toast.makeText(requireContext(),m_address,Toast.LENGTH_LONG).show()
                    // Cancel discovery because it otherwise slows down the connection.
                    mBtAdapter?.cancelDiscovery()
                    val device: BluetoothDevice = mBtAdapter.getRemoteDevice(m_address)
                    m_bluetoothSocket = device.createInsecureRfcommSocketToServiceRecord(m_myUUID)
                    //GlobalScope.launch {
                    m_bluetoothSocket!!.connect()
                    //}


                    MQTTConfigFragment().desconexionMQTT()
                    Log.i("MQTT", "DESCONECTADO")
                    m_isConnected = true
                    InitialFragment.MQTToBT = false
                    Log.i("Bluetooth", "CONECTADO")

                    //Para poder luego usarla en el InitialFragment
                    Log.d("Bluetooth", "Direccion guardada en el preferences: ${m_address}")
                    BTpreferences.edit().putString("direccionBTGuardada", m_address).apply()


                    val textView = binding.textView
                    textView.text = "Estableciendo conexión..."

                    Log.d("Bluetooth", "UUID:${m_myUUID}")
                    Log.d("Bluetooth", "Direccion:${m_address}")

                    Toast.makeText(requireContext(),"CONEXION EXITOSA",Toast.LENGTH_LONG).show()
                    Log.i("Bluetooth", "CONEXION EXITOSA")

                    //Para volver al fragment_initial en lugar de con un Intent
                    val fragmentManager = requireActivity().supportFragmentManager
                    val transaction = fragmentManager.beginTransaction()
                    transaction.replace(R.id.fragment_container, InitialFragment())
                    transaction.addToBackStack(null) // Si deseas agregar la transacción a la pila de retroceso
                    transaction.commit()
                } else {
                    if(m_isConnected)
                        Toast.makeText(requireContext(),"YA ESTÁ CONECTADO",Toast.LENGTH_LONG).show()
                    if(idSpinDisp.selectedItemPosition==-1)
                        Toast.makeText(requireContext(),"NO HAY DISPOSITIVO SELECCIONADO",Toast.LENGTH_LONG).show()
                    Log.i("Bluetooth", "O YA ESTÁ CONECTADO, O NO HAY DISPOSITIVO SELECCIONADO")
                }
            } catch (e: IOException) {
                //connectSuccess = false
                e.printStackTrace()
                Toast.makeText(requireContext(),"ERROR DE CONEXION",Toast.LENGTH_LONG).show()
                Log.i("Bluetooth", e.toString())
            }
        }









////
//        // Obtener una instancia de SharedPreferences para guardar la direccion introducida
//        val sharedPreferences = requireContext().getSharedPreferences("MiAppPreferences", Context.MODE_PRIVATE)
//
//        // Recuperar el string almacenado y establecerlo en el editText en la siguiente ejecución de la aplicación
//        val direccionGuardada = sharedPreferences.getString("direccionGuardada", "")
//        Log.d("SharedPreferences", "Direccion guardada: $direccionGuardada")
//        binding.editText1.setText(direccionGuardada)
//
//        // Guardar el string en SharedPreferences cuando el usuario introduzca un texto
//        binding.editText1.addTextChangedListener(object : TextWatcher {
//            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
//            }
//
//            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
//            }
//
//            override fun afterTextChanged(s: Editable?) {
//                val texto = s.toString()
//                val editor = sharedPreferences.edit()
//                editor.putString("direccionGuardada", texto)
//                editor.apply()
//                Log.d("SharedPreferences", "Direccion cambiada: $texto")
//            }
//        })
//
//
//        // Obtener una instancia de SharedPreferences para guardar la direccion introducida
//        //val sharedPreferencesPuerto = requireContext().getSharedPreferences("PuertoPreferences", Context.MODE_PRIVATE)
//
//        // Recuperar el string almacenado y establecerlo en el editText en la siguiente ejecución de la aplicación
//        val puertoGuardado = sharedPreferences.getString("puertoGuardado", "")
//        Log.d("SharedPreferences", "Puerto guardado: $puertoGuardado")
//        binding.editText2.setText(puertoGuardado)
//
//        // Guardar el string en SharedPreferences cuando el usuario introduzca un texto
//        binding.editText2.addTextChangedListener(object : TextWatcher {
//            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
//            }
//
//            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
//            }
//
//            override fun afterTextChanged(s: Editable?) {
//                val texto = s.toString()
//                val editor = sharedPreferences.edit()
//                editor.putString("puertoGuardado", texto)
//                editor.apply()
//                Log.d("SharedPreferences", "Puerto cambiado: $texto")
//            }
//        })
//
//        // Recuperar el string almacenado y establecerlo en el editText en la siguiente ejecución de la aplicación
//        val usuarioGuardado = sharedPreferences.getString("usuarioGuardado", "")
//        Log.d("SharedPreferences", "Usuario guardado: $usuarioGuardado")
//        binding.editTextUsuarioMQTT.setText(usuarioGuardado)
//
//        // Guardar el string en SharedPreferences cuando el usuario introduzca un texto
//        binding.editTextUsuarioMQTT.addTextChangedListener(object : TextWatcher {
//            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
//            }
//
//            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
//            }
//
//            override fun afterTextChanged(s: Editable?) {
//                val texto = s.toString()
//                val editor = sharedPreferences.edit()
//                editor.putString("usuarioGuardado", texto)
//                editor.apply()
//                Log.d("SharedPreferences", "Usuario cambiado: $texto")
//            }
//        })
//
//
//        // Recuperar el string almacenado y establecerlo en el editText en la siguiente ejecución de la aplicación
//        val contrasenaGuardado = sharedPreferences.getString("contrasenaGuardado", "")
//        Log.d("SharedPreferences", "Contraseña guardado: $contrasenaGuardado")
//        binding.editTextContrasenaMQTT.setText(contrasenaGuardado)
//
//        // Guardar el string en SharedPreferences cuando el usuario introduzca un texto
//        binding.editTextContrasenaMQTT.addTextChangedListener(object : TextWatcher {
//            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
//            }
//
//            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
//            }
//
//            override fun afterTextChanged(s: Editable?) {
//                val texto = s.toString()
//                val editor = sharedPreferences.edit()
//                editor.putString("contrasenaGuardado", texto)
//                editor.apply()
//                Log.d("SharedPreferences", "Contraseña cambiado: $texto")
//            }
//        })
//
////
//        //Esta línea de código determina si el dispositivo es Tablet o Móvil (fundamental para el dibujado correcto y el layout)
//        val isTablet = requireContext().resources.configuration.screenLayout and Configuration.SCREENLAYOUT_SIZE_MASK >= Configuration.SCREENLAYOUT_SIZE_LARGE
//
//
//
//        binding.buttonTopics.setOnClickListener {
//            //Código para crear y mostrar la ventana emergente(AlertDialog)
//            val builder = AlertDialog.Builder(requireActivity())
//            builder.setTitle("Introduzca al menos 1 tópico para publicar")
//            //var bindingDialogTopics = DialogTopicsBinding.inflate(inflater)
//            val viewDialog = layoutInflater.inflate(R.layout.dialog_topics, null)
//
//            val topico1 = viewDialog.findViewById<EditText>(R.id.editTextTopic1)
//            val topico2 = viewDialog.findViewById<EditText>(R.id.editTextTopic2)
//            val topico3 = viewDialog.findViewById<EditText>(R.id.editTextTopic3)
//            val topico4 = viewDialog.findViewById<EditText>(R.id.editTextTopic4)
//            val topico5 = viewDialog.findViewById<EditText>(R.id.editTextTopic5)
//            val botonFinalizar = viewDialog.findViewById<Button>(R.id.buttonFinalizar)
//
//            builder.setView(viewDialog)
//            val dialog = builder.create()
//            dialog.show()
//
//            if(isTablet)
//                dialog.getWindow()?.setLayout(1080, 1150)
//            else
//                dialog.getWindow()?.setLayout(1080, 1500)
//
//
////
//            // Obtener una instancia de SharedPreferences para guardar la direccion introducida
//            val sharedPreferencesTopic = requireContext().getSharedPreferences("TopicPreferences", Context.MODE_PRIVATE)
//
//            // Recuperar el string almacenado y establecerlo en el editText en la siguiente ejecución de la aplicación
//            val topico1Guardado = sharedPreferencesTopic.getString("topico1Guardado", "")
//            topico1.setText(topico1Guardado)
//
//            // Guardar el string en SharedPreferences cuando el usuario introduzca un texto
//            topico1.addTextChangedListener(object : TextWatcher {
//                override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
//                }
//
//                override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
//                }
//
//                override fun afterTextChanged(s: Editable?) {
//                    val texto = s.toString()
//                    val editor = sharedPreferencesTopic.edit()
//                    editor.putString("topico1Guardado", texto)
//                    editor.apply()
//                    Log.d("SharedPreferences", "Topico1 cambiado: $texto")
//                }
//            })
//
//            // Recuperar el string almacenado y establecerlo en el editText en la siguiente ejecución de la aplicación
//            val topico2Guardado = sharedPreferencesTopic.getString("topico2Guardado", "")
//            topico2.setText(topico2Guardado)
//
//            // Guardar el string en SharedPreferences cuando el usuario introduzca un texto
//            topico2.addTextChangedListener(object : TextWatcher {
//                override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
//                }
//
//                override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
//                }
//
//                override fun afterTextChanged(s: Editable?) {
//                    val texto = s.toString()
//                    val editor = sharedPreferencesTopic.edit()
//                    editor.putString("topico2Guardado", texto)
//                    editor.apply()
//                    Log.d("SharedPreferences", "Topico2 cambiado: $texto")
//                }
//            })
//
//            // Recuperar el string almacenado y establecerlo en el editText en la siguiente ejecución de la aplicación
//            val topico3Guardado = sharedPreferencesTopic.getString("topico3Guardado", "")
//            topico3.setText(topico3Guardado)
//
//            // Guardar el string en SharedPreferences cuando el usuario introduzca un texto
//            topico3.addTextChangedListener(object : TextWatcher {
//                override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
//                }
//
//                override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
//                }
//
//                override fun afterTextChanged(s: Editable?) {
//                    val texto = s.toString()
//                    val editor = sharedPreferencesTopic.edit()
//                    editor.putString("topico3Guardado", texto)
//                    editor.apply()
//                    Log.d("SharedPreferences", "Topico3 cambiado: $texto")
//                }
//            })
////
//            // Recuperar el string almacenado y establecerlo en el editText en la siguiente ejecución de la aplicación
//            val topico4Guardado = sharedPreferencesTopic.getString("topico4Guardado", "")
//            topico4.setText(topico4Guardado)
//
//            // Guardar el string en SharedPreferences cuando el usuario introduzca un texto
//            topico4.addTextChangedListener(object : TextWatcher {
//                override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
//                }
//
//                override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
//                }
//
//                override fun afterTextChanged(s: Editable?) {
//                    val texto = s.toString()
//                    val editor = sharedPreferencesTopic.edit()
//                    editor.putString("topico4Guardado", texto)
//                    editor.apply()
//                    Log.d("SharedPreferences", "Topico4 cambiado: $texto")
//                }
//            })
//
//            val topico5Guardado = sharedPreferencesTopic.getString("topico5Guardado", "")
//            topico5.setText(topico5Guardado)
//
//            topico5.addTextChangedListener(object : TextWatcher {
//                override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
//                }
//
//                override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
//                }
//
//                override fun afterTextChanged(s: Editable?) {
//                    val texto = s.toString()
//                    val editor = sharedPreferencesTopic.edit()
//                    editor.putString("topico5Guardado", texto)
//                    editor.apply()
//                    Log.d("SharedPreferences", "Topico5 cambiado: $texto")
//                }
//            })
////
//
//
//            botonFinalizar.setOnClickListener{
//                if(topico1.text.toString() == "")
//                    Toast.makeText(requireContext(), "Debe haber al menos un tópico para publicar", Toast.LENGTH_LONG).show()
//                else{
//                    MQTTConfigFragment.publishTopic1 = topico1.text.toString()
//                    if(topico2.text.toString() != ""){
//                        MQTTConfigFragment.publishTopic2 = topico2.text.toString()
//                    }
//                    if(topico3.text.toString() != ""){
//                        MQTTConfigFragment.publishTopic3 = topico3.text.toString()
//                    }
//                    if(topico4.text.toString() != ""){
//                        MQTTConfigFragment.publishTopic4 = topico4.text.toString()
//                    }
//                    if(topico5.text.toString() != ""){
//                        MQTTConfigFragment.publishTopic5 = topico5.text.toString()
//                    }
//                    dialog.dismiss()
//                }
//            }
//        }
//
//        binding.button.setOnClickListener {
//            MQTTConfigFragment.direccion = binding.editText1.text.toString()
//            MQTTConfigFragment.puerto = binding.editText2.text.toString()
//            MQTTConfigFragment.usuarioMQTT = binding.editTextUsuarioMQTT.text.toString()
//            MQTTConfigFragment.contrasenaMQTT = binding.editTextContrasenaMQTT.text.toString()
//
//
//            if(MQTTConfigFragment.direccion.isEmpty() || MQTTConfigFragment.puerto.isEmpty()){
//                Toast.makeText(requireContext(), "Los campos no pueden estar vacíos", Toast.LENGTH_SHORT).show()
//            }else{
//                Toast.makeText(requireContext(), "Intentando conexión", Toast.LENGTH_LONG).show()
//                try{
//                    MQTTConfigFragment.serverUri = "tcp://${MQTTConfigFragment.direccion}:${MQTTConfigFragment.puerto}"
//
//                    Log.d("Servidor" ,"ServerUri = ${MQTTConfigFragment.serverUri}")
//
//                    conexionMQTT(requireContext(), MQTTConfigFragment.serverUri){ exitosa ->
//                        if(exitosa){
//                            val fragmentManager = requireActivity().supportFragmentManager
//                            val transaction = fragmentManager.beginTransaction()
//                            transaction.replace(R.id.fragment_container, InitialFragment())
//                            transaction.addToBackStack(null) // Si deseas agregar la transacción a la pila de retroceso
//                            transaction.commit()
//                        } else {
//                            val handler = Handler(Looper.getMainLooper())
//                            handler.post {
//                                Toast.makeText(requireContext(), "Conexión fallida, vuelve a intentarlo", Toast.LENGTH_LONG).show()
//                            }
//                        }
//                    }
//                } catch (e: Exception) {
//                    Log.e(MQTTConfigFragment.TAG, "Error al intentar establecer la conexión MQTT.${e.message}")
//                    val handler = Handler(Looper.getMainLooper())
//                    handler.post {
//                        Toast.makeText(requireContext(), "Error al intentar establecer la conexión MQTT: ${e.message}", Toast.LENGTH_LONG).show()
//                    }
//                }
//            }
//        }
    }


    //PARA ENVIAR MENSAJES POR BLUETOOTH
    private fun sendCommand(input: String) {
        if (m_bluetoothSocket != null) {
            try{
                m_bluetoothSocket!!.outputStream.write(input.toByteArray())
            } catch(e: IOException) {
                e.printStackTrace()
            }
        }
    }

    //Para desconectarse
    fun disconnect() {
        try {
            m_bluetoothSocket?.close()
            m_bluetoothSocket = null
            m_isConnected = false
            Log.i("Bluetooth","DESCONECTADO BT")
        } catch (e: IOException) {
            e.printStackTrace()
            Log.i("Bluetooth", "ERROR DURANTE LA DECONEXIÓN")
        }
    }


    //PERMISOS NECESARIOS PARA PODER USAR BLUETOOTH
    private fun permisos() {

        if (ContextCompat.checkSelfPermission(
                requireContext(),
                Manifest.permission.READ_SMS
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            ActivityCompat.requestPermissions(
                requireActivity(), arrayOf(
                    Manifest.permission.BLUETOOTH,
                    Manifest.permission.BLUETOOTH_CONNECT,
                    Manifest.permission.BLUETOOTH_ADMIN,
                    Manifest.permission.BLUETOOTH_SCAN,
                    Manifest.permission.BLUETOOTH_ADVERTISE,
                    Manifest.permission.BLUETOOTH_PRIVILEGED,
                    Manifest.permission.MANAGE_DEVICE_POLICY_BLUETOOTH,
                ),
                1
            )
        } else {
            Log.d("Bluetooth", "Todos los permisos concedidos ")
        }
    }
}