package com.example.domticacongestosfaciales

import android.Manifest
import android.R
import android.animation.ObjectAnimator
import android.annotation.SuppressLint
import android.bluetooth.BluetoothDevice
import android.bluetooth.BluetoothManager
import android.bluetooth.BluetoothSocket
import android.content.Context
import android.content.pm.PackageManager
import android.content.res.Configuration
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.Path
import android.graphics.Rect
import android.graphics.drawable.BitmapDrawable
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Switch
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.annotation.RequiresApi
import androidx.appcompat.app.AppCompatActivity
import androidx.camera.core.CameraSelector
import androidx.camera.core.ImageAnalysis
import androidx.camera.core.ImageAnalysis.STRATEGY_KEEP_ONLY_LATEST
import androidx.camera.core.ImageProxy
import androidx.camera.core.Preview
import androidx.camera.lifecycle.ProcessCameraProvider
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.example.domticacongestosfaciales.databinding.AppBarMainBinding
import com.example.domticacongestosfaciales.databinding.FragmentInitialBinding
import com.example.domticacongestosfaciales.sqlite.ConexionBBDDSQLite
import com.google.mlkit.vision.common.InputImage
import com.google.mlkit.vision.facemesh.FaceMeshDetection
import com.google.mlkit.vision.facemesh.FaceMeshDetector
import com.google.mlkit.vision.facemesh.FaceMeshDetectorOptions
import com.google.mlkit.vision.facemesh.FaceMeshPoint
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import org.eclipse.paho.client.mqttv3.IMqttActionListener
import org.eclipse.paho.client.mqttv3.IMqttToken
import org.eclipse.paho.client.mqttv3.MqttException
import org.eclipse.paho.client.mqttv3.MqttMessage
import org.json.JSONObject
import java.io.IOException
import java.lang.reflect.Method
import java.util.concurrent.ExecutorService
import java.util.concurrent.Executors
import kotlin.math.sqrt


class InitialFragment : Fragment() {


    //Es companion para poder cambiarlo en MQTTConfigFragment y BTConfigFragment
    companion object{
        //Para poder alternar entre conectar MQTT y BT
        var MQTToBT: Boolean = false //True = MQTT el último conectado, False = Bt último conectado
    }


    /*
    Variables que creé para intentar dibujar las flechas que se dibujan sobre la cara por segmentos

    Lo intenté para mostrar al usuario lo que quedaba para completar el envío de mensajes MQTT o BT
    Pero al final desistí porque no se dibujaban bien, a veces parpadeaban las flechas cuando
    no debían
     */
    var dibujarSeg1:Boolean = false
    var dibujarSeg2:Boolean = false

    //Como intento para que detecte si el usuario se duerme y no haga nada
    var dormido = false



    //Variables globales
    private lateinit var binding: FragmentInitialBinding
    private lateinit var cameraExecutor: ExecutorService
    private lateinit var defaultDetector: FaceMeshDetector

    //Un handler para cada gesto para evitar que cada vez tarde un tiempo distinto en detectar el gesto
    private var handlerCejas = Handler(Looper.getMainLooper())
    private var handlerBoca = Handler(Looper.getMainLooper())
    private var handlerPosBoca = Handler(Looper.getMainLooper())
    private var handlerOjoDer = Handler(Looper.getMainLooper())
    private var handlerOjoIzq = Handler(Looper.getMainLooper())

    private var handlerOjos = Handler(Looper.getMainLooper())

    //Handlers que creé para intentar dibujar las flechas por segmentos
   // private var handlerFlechaCejas1 = Handler(Looper.getMainLooper())
   // private var handlerFlechaCejas2 = Handler(Looper.getMainLooper())
   // private var handlerFlechaCejas3 = Handler(Looper.getMainLooper())
   // private var handlerFlechaCejas4 = Handler(Looper.getMainLooper())


    //Al ser fragment y no activity, uso esto
    private var mActivity: AppCompatActivity? = null

    //Para usar la cámara en la función startCamera
    private var cameraProvider: ProcessCameraProvider? = null

    //Para evitar que parpadee la lámpara 80 veces al subir cejas y abrir boca a la vez
    //private var ultimoApagado: Long = 0
    //private var ultimoEncendido: Long = 0
    private var ultimoEnviado: Long = 0
    private var ultimoBT: Long = 0
    private var ultimaConexion: Long = 0
    private var ultimoCambio: Long = 0
    //private var transcurridoDesdeEncendido: Long = 0
    //private var transcurridoDesdeApagado: Long = 0
    private var transcurridoDesdeEnviado: Long = 0
    private var transcurridoDesdeBT: Long = 0
    private var transcurridoDesdeConexion: Long = 0
    private var transcurridoDesdeCambio: Long = 0

    //Variable para cambiar el tópico en el que publicar con MQTT
    var topicoActual: Int = 1

    //Para comprobar que no se enviaran 500 ON o 500 OFF al broker
    var lastPublishedMessage: String? = null



    //Para evitar el error de "fragment not attached to Activity"
    override fun onAttach(context: Context) {
        super.onAttach(context)
        if (context is AppCompatActivity) {
            mActivity = context
        } else {
            throw RuntimeException("$context must implement AppCompatActivity")
        }
    }

    //Función que asocia todos los elementos del layout con el código mediante binding
    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        binding = FragmentInitialBinding.inflate(inflater,container,false)
        return binding.root
    }

    //Función de ejecución del programa (de ejecución de este fragment más bien)
    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        cameraExecutor = Executors.newSingleThreadExecutor()
        requestPermission()
        defaultDetector = FaceMeshDetection.getClient(FaceMeshDetectorOptions.Builder()
            .build())
    }

    //Función para cuando se cierra el fragment
    override fun onDestroyView() {
        super.onDestroyView()
        // Liberar recursos al cambiar de fragmento
        cameraProvider?.unbindAll()
        defaultDetector.close()
        cameraExecutor.shutdown()
    }

    //Para pedir el permiso de acceso a cámara la primera vez que se usa la aplicación
    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
    private fun requestPermission(){
        requestPermissionIfMissing{granted->
            if(granted)
                startCamera()
            else
                Toast.makeText(requireContext(),"Por favor, acepta la solicitud", Toast.LENGTH_LONG).show()
        }
    }

    //Para volver a pedir permiso si el usuario dice que no
    private fun requestPermissionIfMissing(onResult:((Boolean)->Unit)){
        if(ContextCompat.checkSelfPermission(requireContext(),Manifest.permission.CAMERA) == PackageManager.PERMISSION_GRANTED)
            onResult(true)
        else
            registerForActivityResult(ActivityResultContracts.RequestPermission()){
                onResult(it)
            }.launch(Manifest.permission.CAMERA)
    }

    //Para usar la cámara del dispositivo
    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
    private fun startCamera(){
        if(!isAdded){
            return
        }
        val processCameraProvider = ProcessCameraProvider.getInstance(requireContext())
        processCameraProvider.addListener({
            try {
                if(!isAdded){
                    return@addListener
                }
                val cameraProvider = processCameraProvider.get()
                val previewUseCase = buildPreviewUseCase()
                val imageAnalysisUseCase = buildImageAnalysisUseCase()
                cameraProvider.unbindAll()
                cameraProvider.bindToLifecycle(this, CameraSelector.DEFAULT_FRONT_CAMERA,previewUseCase, imageAnalysisUseCase)
            } catch (e: Exception) {
            }
        },ContextCompat.getMainExecutor(requireContext()))
    }

    //Para poder mostrar por pantalla la vista de la cámara, y para usar esa vista al dibujar
    private fun buildPreviewUseCase(): Preview {
        return Preview.Builder().build().also{it.setSurfaceProvider(binding.previewView.surfaceProvider)}
    }

    //Función en la que hago todos los cálculos y detección de gestos
    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
    private fun buildImageAnalysisUseCase(): ImageAnalysis {

//Variables comunes a la función a usar

        //Lista de posiciones X e Y de los puntos faciales detectados por ML Kit
        val posicionesX:MutableList<Float> = mutableListOf()
        val posicionesY:MutableList<Float> = mutableListOf()

        //Variables para determinar si se realiza un gesto (0=sí, 1=no (para cejas y boca al menos))
        //No puse booleanas para poder usar PosBoca en la misma función de "determinarGestos"
        var cejas:Int
        var boca:Int
        var posBoca:Int
        var ojoDer:Int
        var ojoIzq:Int

        //Para dibujar también por pantalla el texto del gesto realizado en caso de activar vista para daltónicos
        var textoAMostrar:String = "Si visualizar texto no está activo, no se debería ver este texto"


        //Voy a probar a actualizar los valores de los topics para que no publique en AndroidADomotica2
        //val topicPreferences = requireContext().getSharedPreferences("MiAppPreferences", Context.MODE_PRIVATE)
        val dbHelper = ConexionBBDDSQLite(requireContext())
//        val topico1Guardado = dbHelper.getTopicById(1)
//        MQTTConfigFragment.publishTopic1 = topico1Guardado.toString()
//        val topico2Guardado = dbHelper.getTopicById(2)
//        MQTTConfigFragment.publishTopic2 = topico2Guardado.toString()
//        val topico3Guardado = dbHelper.getTopicById(3)
//        MQTTConfigFragment.publishTopic3 = topico3Guardado.toString()
//        val topico4Guardado = dbHelper.getTopicById(4)
//        MQTTConfigFragment.publishTopic4 = topico4Guardado.toString()
//        val topico5Guardado = dbHelper.getTopicById(5)
//        MQTTConfigFragment.publishTopic5 = topico5Guardado.toString()



        //
        //Intento que el número de líneas de la base de topics sea siempre el número de topics
        //Como no se actualiza hasta que no se quita el focus de un editText, suele aperecer con
        //un valor menos de lo real, así que lo voy a mover al initialFragment
        MQTTConfigFragment.numeroTopics = dbHelper.check_t_topics()
        Log.d("Base datos Topicos", "Numero topicos = ${MQTTConfigFragment.numeroTopics}")
//



//
        //Para guardar el último topico seleccionado
        val preferencesTopicoActual = requireContext().getSharedPreferences("TopicoActualPreferences", Context.MODE_PRIVATE)

        //Recuperar el último tópico y comprobar si sigue existiendo ese número de topic
        var checkTopicActual = preferencesTopicoActual.getInt("topicoActual", 1)

        if(checkTopicActual<=ConexionBBDDSQLite(requireContext()).check_t_topics()){
            topicoActual = checkTopicActual
        } else {
            topicoActual = 1
        }
//



//Para hacer que al arrancar la aplicación se carguen los últimos gestos seleccionados
        //Para guardar la configuración de gestos entre ejecución y ejecución por así decirlo
        val preferences = requireContext().getSharedPreferences("UmbralesPreferences", Context.MODE_PRIVATE)

        //Recuperar el estado de los switches de SharedPreferences
        val estadoSwitch1 = preferences.getBoolean("switch1Guardado", true)
        val estadoSwitch2 = preferences.getBoolean("switch2Guardado", false)
        val estadoSwitchConexionCejas = preferences.getBoolean("switchConexionCejasGuardado", false)
        val estadoSwitch3 = preferences.getBoolean("switch3Guardado", false)
        val estadoSwitch4 = preferences.getBoolean("switch4Guardado", true)
        val estadoSwitchConexionBoca = preferences.getBoolean("switchConexionBocaGuardado", false)
        val estadoSwitch5 = preferences.getBoolean("switch5Guardado", false)
        val estadoSwitch6 = preferences.getBoolean("switch6Guardado", false)
        val estadoSwitchConexionOjoDer = preferences.getBoolean("switchConexionOjoDerGuardado", true)
        val estadoSwitch7 = preferences.getBoolean("switch7Guardado", false)
        val estadoSwitch8 = preferences.getBoolean("switch8Guardado", false)
        val estadoSwitchConexionOjoIzq = preferences.getBoolean("switchConexionOjoIzqGuardado", false)


        //Intento de hacer que coincidan los gestos con los switches aún al arrancar la aplicación
        //Se ponen los gestos como el último estado guardado
        // Inicializar las variables de acción basadas en los estados recuperados
        UsageInfoFragment.cejasEncender = estadoSwitch1
        UsageInfoFragment.cejasApagar = estadoSwitch2
        UsageInfoFragment.cejasConexion = estadoSwitchConexionCejas
        UsageInfoFragment.bocaEncender = estadoSwitch3
        UsageInfoFragment.bocaApagar = estadoSwitch4
        UsageInfoFragment.bocaConexion = estadoSwitchConexionBoca
        UsageInfoFragment.ojoDerEncender = estadoSwitch5
        UsageInfoFragment.ojoDerApagar = estadoSwitch6
        UsageInfoFragment.ojoDerConexion = estadoSwitchConexionOjoDer
        UsageInfoFragment.ojoIzqEncender = estadoSwitch7
        UsageInfoFragment.ojoIzqApagar = estadoSwitch8
        UsageInfoFragment.ojoIzqConexion = estadoSwitchConexionOjoIzq


        //Ahora lo mismo que con los switches pero para las seekBars de los umbrales
        val estadoUmbralCejas = preferences.getInt("umbralCejasGuardado", 14)
        val estadoUmbralBoca = preferences.getInt("umbralBocaGuardado", 8)
        val estadoUmbralOjoDer = preferences.getInt("umbralOjoDerGuardado", 3)
        val estadoUmbralOjoIzq = preferences.getInt("umbralOjoIzqGuardado", 3)

        UsageInfoFragment.umbralCejas = estadoUmbralCejas
        UsageInfoFragment.umbralBoca = estadoUmbralBoca
        UsageInfoFragment.umbralOjoDer = estadoUmbralOjoDer
        UsageInfoFragment.umbralOjoIzq = estadoUmbralOjoIzq



        //Resultado de la función (lo que se devuelve)(lo que se ejecuta al invocarla)
        return ImageAnalysis.Builder()
            .setBackpressureStrategy(STRATEGY_KEEP_ONLY_LATEST)
            .build().also{ analysis ->
                analysis.setAnalyzer(cameraExecutor,object: ImageAnalysis.Analyzer{
                    @SuppressLint("UnsafeOptInUsageError")
                    override fun analyze(image: ImageProxy) {
                        //Este if era por no dejarme cambiar de framento al no estar vinculado con Activity
                        if (!isAdded()) {
                            // El fragmento no está adjunto a una actividad, no hagas nada
                            return
                        }

                        //Para mostrar o ocultar la interfaz en el móvil
                        requireActivity().runOnUiThread {
                            if (!UsageInfoFragment.visualizarInterfaz) {
                                binding.linearLayoutPrincipal.visibility = View.GONE
                            } else {
                                binding.linearLayoutPrincipal.visibility = View.VISIBLE
                            }

                            if (!UsageInfoFragment.visualizarTextoUmbrales) {
                                binding.linearLayoutUmbrales?.visibility = View.GONE
                            } else {
                                binding.linearLayoutUmbrales?.visibility = View.VISIBLE
                            }
                        }

//                        mActivity?.runOnUiThread {
//                            //Para que no se vean los umbrales en vertical
//                            if(resources.configuration.orientation == Configuration.ORIENTATION_PORTRAIT){
//                                binding.linearLayoutUmbrales?.visibility = View.GONE
//                            }
//                        }




                        //Para actualizar el textView de conexión(debajo de la barra azul)
                        //He puesto el && para asegurarme de que no estén conectados a la vez
                        if(MQTTConfigFragment.conectado && !BTConfigFragment.m_isConnected){
                            mActivity?.runOnUiThread{
                                binding.textViewConect.text = "CONECTADO AL SERVIDOR MQTT"
                                binding.iconoConectado.visibility = View.VISIBLE
                                binding.iconoDesconectado.visibility = View.INVISIBLE
                                binding.iconoMQTT.visibility = View.VISIBLE
                                binding.iconoBluetooth.visibility = View.INVISIBLE
                            }
                        }
                        else if(BTConfigFragment.m_isConnected && !MQTTConfigFragment.conectado) {
                            mActivity?.runOnUiThread {
                                binding.textViewConect.text = "CONECTADO POR BT"
                                binding.iconoConectado.visibility = View.VISIBLE
                                binding.iconoDesconectado.visibility = View.INVISIBLE
                                binding.iconoMQTT.visibility = View.INVISIBLE
                                binding.iconoBluetooth.visibility = View.VISIBLE
                            }
                        }
                        else if(BTConfigFragment.m_isConnected && MQTTConfigFragment.conectado){
                            mActivity?.runOnUiThread {
                            binding.textViewConect.text = "CONECTADOS AMBOS (NO DEBERÍA PASAR)"
                            }
                        } else {
                            mActivity?.runOnUiThread {
                                binding.textViewConect.text = "NO CONECTADO AL SERVIDOR MQTT NI POR BT"
                                binding.iconoConectado.visibility = View.INVISIBLE
                                binding.iconoDesconectado.visibility = View.VISIBLE
                                binding.iconoMQTT.visibility = View.INVISIBLE
                                binding.iconoBluetooth.visibility = View.INVISIBLE
                            }
                        }

                        //Para actualizar el textView del set de gestos seleccionados
                        if(UsageInfoFragment.cejasEncender && UsageInfoFragment.bocaApagar && UsageInfoFragment.ojoDerConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = true
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON CEJAS(ENCENDER), BOCA(APAGAR), OJO DERECHO(Conexion) Y OJO IZQUIERDO(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.cejasEncender && UsageInfoFragment.bocaApagar && UsageInfoFragment.ojoIzqConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = true
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON CEJAS(ENCENDER), BOCA(APAGAR), OJO IZQUIERDO(Conexion) Y OJO DERECHO(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.cejasEncender && UsageInfoFragment.ojoDerApagar && UsageInfoFragment.bocaConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = true
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON CEJAS(ENCENDER), OJO DERECHO(APAGAR), BOCA(Conexion) Y OJO IZQUIERDO(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.cejasEncender && UsageInfoFragment.ojoDerApagar && UsageInfoFragment.ojoIzqConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = true
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON CEJAS(ENCENDER), OJO DERECHO(APAGAR), OJO IZQUIERDO(Conexion) Y BOCA(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.cejasEncender && UsageInfoFragment.ojoIzqApagar && UsageInfoFragment.bocaConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = true
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON CEJAS(ENCENDER), OJO IZQUIERDO(APAGAR), BOCA(Conexion) Y OJO DERECHO(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.cejasEncender && UsageInfoFragment.ojoIzqApagar && UsageInfoFragment.ojoDerConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = true
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON CEJAS(ENCENDER), OJO IZQUIERDO(APAGAR), OJO DERECHO(Conexion) Y BOCA(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.bocaEncender && UsageInfoFragment.cejasApagar && UsageInfoFragment.ojoDerConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = true
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON BOCA(ENCENDER), CEJAS(APAGAR), OJO DERECHO(Conexion) Y OJO IZQUIERDO(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.bocaEncender && UsageInfoFragment.cejasApagar && UsageInfoFragment.ojoIzqConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = true
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON BOCA(ENCENDER), CEJAS(APAGAR), OJO IZQUIERDO(Conexion) Y OJO DERECHO(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.bocaEncender && UsageInfoFragment.ojoDerApagar && UsageInfoFragment.cejasConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = true
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON BOCA(ENCENDER), OJO DERECHO(APAGAR), CEJAS(Conexion) Y OJO IZQUIERDO(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.bocaEncender && UsageInfoFragment.ojoDerApagar && UsageInfoFragment.ojoIzqConexion){
                            UsageInfoFragment.cejasCambiar = true
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON BOCA(ENCENDER), OJO DERECHO(APAGAR), OJO IZQUIERDO(Conexion) Y CEJAS(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.bocaEncender && UsageInfoFragment.ojoIzqApagar && UsageInfoFragment.cejasConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = true
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON BOCA(ENCENDER), OJO IZQUIERDO(APAGAR), CEJAS(Conexion) Y OJO DERECHO(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.bocaEncender && UsageInfoFragment.ojoIzqApagar && UsageInfoFragment.ojoDerConexion){
                            UsageInfoFragment.cejasCambiar = true
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON BOCA(ENCENDER), OJO IZQUIERDO(APAGAR), OJO DERECHO(Conexion) Y CEJAS(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.ojoDerEncender && UsageInfoFragment.cejasApagar && UsageInfoFragment.bocaConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = true
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON OJO DERECHO(ENCENDER), CEJAS(APAGAR), BOCA(Conexion) Y OJO IZQUIERDO(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.ojoDerEncender && UsageInfoFragment.cejasApagar && UsageInfoFragment.ojoIzqConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = true
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON OJO DERECHO(ENCENDER), CEJAS(APAGAR), OJO IZQUIERDO(Conexion) Y BOCA(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.ojoDerEncender && UsageInfoFragment.bocaApagar && UsageInfoFragment.cejasConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = true
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON OJO DERECHO(ENCENDER), BOCA(APAGAR), CEJAS(Conexion) Y OJO IZQUIERDO(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.ojoDerEncender && UsageInfoFragment.bocaApagar && UsageInfoFragment.ojoIzqConexion){
                            UsageInfoFragment.cejasCambiar = true
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON OJO DERECHO(ENCENDER), BOCA(APAGAR), OJO IZQUIERDO(Conexion) Y CEJAS(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.ojoDerEncender && UsageInfoFragment.ojoIzqApagar && UsageInfoFragment.cejasConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = true
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON OJO DERECHO(ENCENDER), OJO IZQUIERDO(APAGAR), CEJAS(Conexion) Y BOCA(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.ojoDerEncender && UsageInfoFragment.ojoIzqApagar && UsageInfoFragment.bocaConexion){
                            UsageInfoFragment.cejasCambiar = true
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON OJO DERECHO(ENCENDER), OJO IZQUIERDO(APAGAR), BOCA(Conexion) Y CEJAS(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.ojoIzqEncender && UsageInfoFragment.cejasApagar && UsageInfoFragment.bocaConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = true
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON OJO IZQUIERDO(ENCENDER), CEJAS(APAGAR), BOCA(Conexion) Y OJO DERECHO(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.ojoIzqEncender && UsageInfoFragment.cejasApagar && UsageInfoFragment.ojoDerConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = true
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON OJO IZQUIERDO(ENCENDER), CEJAS(APAGAR), OJO DERECHO(Conexion) Y BOCA(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.ojoIzqEncender && UsageInfoFragment.bocaApagar && UsageInfoFragment.cejasConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = true
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON OJO IZQUIERDO(ENCENDER), BOCA(APAGAR), CEJAS(Conexion) Y OJO DERECHO(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.ojoIzqEncender && UsageInfoFragment.bocaApagar && UsageInfoFragment.ojoDerConexion){
                            UsageInfoFragment.cejasCambiar = true
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread{
                                binding.textViewGestos.text = "CONTROL CON OJO IZQUIERDO(ENCENDER), BOCA(APAGAR), OJO DERECHO(Conexion) Y CEJAS(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.ojoIzqEncender && UsageInfoFragment.ojoDerApagar && UsageInfoFragment.cejasConexion){
                            UsageInfoFragment.cejasCambiar = false
                            UsageInfoFragment.bocaCambiar = true
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread {
                                binding.textViewGestos.text = "CONTROL CON OJO IZQUIERDO(ENCENDER), OJO DERECHO(APAGAR), CEJAS(Conexion) Y BOCA(CAMBIAR TÓPICO)"
                            }
                        }
                        else if(UsageInfoFragment.ojoIzqEncender && UsageInfoFragment.ojoDerApagar && UsageInfoFragment.bocaConexion){
                            UsageInfoFragment.cejasCambiar = true
                            UsageInfoFragment.bocaCambiar = false
                            UsageInfoFragment.ojoDerCambiar = false
                            UsageInfoFragment.ojoIzqCambiar = false
                            mActivity?.runOnUiThread {
                                binding.textViewGestos.text = "CONTROL CON OJO IZQUIERDO(ENCENDER), OJO DERECHO(APAGAR), BOCA(Conexion) Y CEJAS(CAMBIAR TÓPICO)"
                            }
                        } else {
                            mActivity?.runOnUiThread {
                                binding.textViewGestos.text = "HA SUCEDIDO ALGÚN ERROR"
                            }
                        }


                        //Solamente para iconos de gestos
                        if(UsageInfoFragment.cejasEncender){
                            mActivity?.runOnUiThread {
                                binding.iconoCejasEnc.visibility = View.VISIBLE
                                binding.iconoBocaEnc.visibility = View.INVISIBLE
                                binding.iconoOjoDerEnc.visibility = View.INVISIBLE
                                binding.iconoOjoIzqEnc.visibility = View.INVISIBLE
                            }
                        }
                        else if(UsageInfoFragment.bocaEncender){
                            mActivity?.runOnUiThread {
                                binding.iconoCejasEnc.visibility = View.INVISIBLE
                                binding.iconoBocaEnc.visibility = View.VISIBLE
                                binding.iconoOjoDerEnc.visibility = View.INVISIBLE
                                binding.iconoOjoIzqEnc.visibility = View.INVISIBLE
                            }
                        }
                        else if(UsageInfoFragment.ojoDerEncender){
                            mActivity?.runOnUiThread {
                                binding.iconoCejasEnc.visibility = View.INVISIBLE
                                binding.iconoBocaEnc.visibility = View.INVISIBLE
                                binding.iconoOjoDerEnc.visibility = View.VISIBLE
                                binding.iconoOjoIzqEnc.visibility = View.INVISIBLE
                            }
                        }
                        else if(UsageInfoFragment.ojoIzqEncender){
                            mActivity?.runOnUiThread {
                                binding.iconoCejasEnc.visibility = View.INVISIBLE
                                binding.iconoBocaEnc.visibility = View.INVISIBLE
                                binding.iconoOjoDerEnc.visibility = View.INVISIBLE
                                binding.iconoOjoIzqEnc.visibility = View.VISIBLE
                            }
                        }

                        if(UsageInfoFragment.cejasApagar){
                            mActivity?.runOnUiThread {
                                binding.iconoCejasApa.visibility = View.VISIBLE
                                binding.iconoBocaApa.visibility = View.INVISIBLE
                                binding.iconoOjoDerApa.visibility = View.INVISIBLE
                                binding.iconoOjoIzqApa.visibility = View.INVISIBLE
                            }
                        }
                        else if(UsageInfoFragment.bocaApagar){
                            mActivity?.runOnUiThread {
                                binding.iconoCejasApa.visibility = View.INVISIBLE
                                binding.iconoBocaApa.visibility = View.VISIBLE
                                binding.iconoOjoDerApa.visibility = View.INVISIBLE
                                binding.iconoOjoIzqApa.visibility = View.INVISIBLE
                            }
                        }
                        else if(UsageInfoFragment.ojoDerApagar){
                            mActivity?.runOnUiThread {
                                binding.iconoCejasApa.visibility = View.INVISIBLE
                                binding.iconoBocaApa.visibility = View.INVISIBLE
                                binding.iconoOjoDerApa.visibility = View.VISIBLE
                                binding.iconoOjoIzqApa.visibility = View.INVISIBLE
                            }
                        }
                        else if(UsageInfoFragment.ojoIzqApagar){
                            mActivity?.runOnUiThread {
                                binding.iconoCejasApa.visibility = View.INVISIBLE
                                binding.iconoBocaApa.visibility = View.INVISIBLE
                                binding.iconoOjoDerApa.visibility = View.INVISIBLE
                                binding.iconoOjoIzqApa.visibility = View.VISIBLE
                            }
                        }

                        if(UsageInfoFragment.cejasConexion){
                            mActivity?.runOnUiThread {
                                binding.iconoCejasCon.visibility = View.VISIBLE
                                binding.iconoBocaCon.visibility = View.INVISIBLE
                                binding.iconoOjoDerCon.visibility = View.INVISIBLE
                                binding.iconoOjoIzqCon.visibility = View.INVISIBLE
                            }
                        }
                        else if(UsageInfoFragment.bocaConexion){
                            mActivity?.runOnUiThread {
                                binding.iconoCejasCon.visibility = View.INVISIBLE
                                binding.iconoBocaCon.visibility = View.VISIBLE
                                binding.iconoOjoDerCon.visibility = View.INVISIBLE
                                binding.iconoOjoIzqCon.visibility = View.INVISIBLE
                            }
                        }
                        else if(UsageInfoFragment.ojoDerConexion){
                            mActivity?.runOnUiThread {
                                binding.iconoCejasCon.visibility = View.INVISIBLE
                                binding.iconoBocaCon.visibility = View.INVISIBLE
                                binding.iconoOjoDerCon.visibility = View.VISIBLE
                                binding.iconoOjoIzqCon.visibility = View.INVISIBLE
                            }
                        }
                        else if(UsageInfoFragment.ojoIzqConexion){
                            mActivity?.runOnUiThread {
                                binding.iconoCejasCon.visibility = View.INVISIBLE
                                binding.iconoBocaCon.visibility = View.INVISIBLE
                                binding.iconoOjoDerCon.visibility = View.INVISIBLE
                                binding.iconoOjoIzqCon.visibility = View.VISIBLE
                            }
                        }

                        if(UsageInfoFragment.cejasCambiar){
                            mActivity?.runOnUiThread {
                                binding.iconoCejasCam.visibility = View.VISIBLE
                                binding.iconoBocaCam.visibility = View.INVISIBLE
                                binding.iconoOjoDerCam.visibility = View.INVISIBLE
                                binding.iconoOjoIzqCam.visibility = View.INVISIBLE
                            }
                        }
                        else if(UsageInfoFragment.bocaCambiar){
                            mActivity?.runOnUiThread {
                                binding.iconoCejasCam.visibility = View.INVISIBLE
                                binding.iconoBocaCam.visibility = View.VISIBLE
                                binding.iconoOjoDerCam.visibility = View.INVISIBLE
                                binding.iconoOjoIzqCam.visibility = View.INVISIBLE
                            }
                        }
                        else if(UsageInfoFragment.ojoDerCambiar){
                            mActivity?.runOnUiThread {
                                binding.iconoCejasCam.visibility = View.INVISIBLE
                                binding.iconoBocaCam.visibility = View.INVISIBLE
                                binding.iconoOjoDerCam.visibility = View.VISIBLE
                                binding.iconoOjoIzqCam.visibility = View.INVISIBLE
                            }
                        }
                        else if(UsageInfoFragment.ojoIzqCambiar){
                            mActivity?.runOnUiThread {
                                binding.iconoCejasCam.visibility = View.INVISIBLE
                                binding.iconoBocaCam.visibility = View.INVISIBLE
                                binding.iconoOjoDerCam.visibility = View.INVISIBLE
                                binding.iconoOjoIzqCam.visibility = View.VISIBLE
                            }
                        }



//
                        //Esta línea es importante, al introducir el tópico guardado en la DB de topicos en la variable en la que se publican los mensajes
                        MQTTConfigFragment.publishTopic = ConexionBBDDSQLite(requireContext()).getTopicByPosition(topicoActual).toString()
//
                        mActivity?.runOnUiThread {
                            binding.textViewTopicoActual.text = "TÓPICO EN EL QUE SE PUBLICA: $topicoActual (${MQTTConfigFragment.publishTopic})"
                        }


                        //Para los iconos del tópico actual
                        if(topicoActual==1){
                            mActivity?.runOnUiThread {
                                binding.icono1.visibility = View.VISIBLE
                                binding.icono2.visibility = View.INVISIBLE
                                binding.icono3.visibility = View.INVISIBLE
                                binding.icono4.visibility = View.INVISIBLE
                                binding.icono5.visibility = View.INVISIBLE
                                binding.icono6.visibility = View.INVISIBLE
                                binding.iconomas.visibility = View.INVISIBLE
                            }
                        }
                        else if(topicoActual==2){
                            mActivity?.runOnUiThread {
                                binding.icono1.visibility = View.INVISIBLE
                                binding.icono2.visibility = View.VISIBLE
                                binding.icono3.visibility = View.INVISIBLE
                                binding.icono4.visibility = View.INVISIBLE
                                binding.icono5.visibility = View.INVISIBLE
                                binding.icono6.visibility = View.INVISIBLE
                                binding.iconomas.visibility = View.INVISIBLE
                            }
                        }
                        else if(topicoActual==3){
                            mActivity?.runOnUiThread {
                                binding.icono1.visibility = View.INVISIBLE
                                binding.icono2.visibility = View.INVISIBLE
                                binding.icono3.visibility = View.VISIBLE
                                binding.icono4.visibility = View.INVISIBLE
                                binding.icono5.visibility = View.INVISIBLE
                                binding.icono6.visibility = View.INVISIBLE
                                binding.iconomas.visibility = View.INVISIBLE
                            }
                        }
                        else if(topicoActual==4){
                            mActivity?.runOnUiThread {
                                binding.icono1.visibility = View.INVISIBLE
                                binding.icono2.visibility = View.INVISIBLE
                                binding.icono3.visibility = View.INVISIBLE
                                binding.icono4.visibility = View.VISIBLE
                                binding.icono5.visibility = View.INVISIBLE
                                binding.icono6.visibility = View.INVISIBLE
                                binding.iconomas.visibility = View.INVISIBLE
                            }
                        }
                        else if(topicoActual==5){
                            mActivity?.runOnUiThread {
                                binding.icono1.visibility = View.INVISIBLE
                                binding.icono2.visibility = View.INVISIBLE
                                binding.icono3.visibility = View.INVISIBLE
                                binding.icono4.visibility = View.INVISIBLE
                                binding.icono5.visibility = View.VISIBLE
                                binding.icono6.visibility = View.INVISIBLE
                                binding.iconomas.visibility = View.INVISIBLE
                            }
                        }
                        else if(topicoActual==6){
                            mActivity?.runOnUiThread {
                                binding.icono1.visibility = View.INVISIBLE
                                binding.icono2.visibility = View.INVISIBLE
                                binding.icono3.visibility = View.INVISIBLE
                                binding.icono4.visibility = View.INVISIBLE
                                binding.icono5.visibility = View.INVISIBLE
                                binding.icono6.visibility = View.VISIBLE
                                binding.iconomas.visibility = View.INVISIBLE
                            }
                        } else {
                            mActivity?.runOnUiThread {
                                binding.icono1.visibility = View.INVISIBLE
                                binding.icono2.visibility = View.INVISIBLE
                                binding.icono3.visibility = View.INVISIBLE
                                binding.icono4.visibility = View.INVISIBLE
                                binding.icono5.visibility = View.INVISIBLE
                                binding.icono6.visibility = View.INVISIBLE
                                binding.iconomas.visibility = View.VISIBLE
                            }
                        }



                        //Para actualizar el textView del último mensaje enviado y los iconos
                        mActivity?.runOnUiThread{
                            if(lastPublishedMessage==null){
                                binding.textViewUltimoMensaje.text = "Aún no se ha publicado"
                                binding.iconoOn.visibility = View.INVISIBLE
                                binding.iconoOff.visibility = View.INVISIBLE
                                binding.iconoCambioPlato.visibility = View.INVISIBLE
                                binding.iconoComer.visibility = View.INVISIBLE
                            } else {
                                binding.textViewUltimoMensaje.text = "$lastPublishedMessage"
                                if(lastPublishedMessage=="EnviadoON"){
                                    binding.iconoOn.visibility = View.VISIBLE
                                    binding.iconoOff.visibility = View.INVISIBLE
                                    binding.iconoCambioPlato.visibility = View.INVISIBLE
                                    binding.iconoComer.visibility = View.INVISIBLE
                                }
                                else if(lastPublishedMessage=="EnviadoOFF"){
                                    binding.iconoOn.visibility = View.INVISIBLE
                                    binding.iconoOff.visibility = View.VISIBLE
                                    binding.iconoCambioPlato.visibility = View.INVISIBLE
                                    binding.iconoComer.visibility = View.INVISIBLE
                                }
                                else if(lastPublishedMessage=="Cambio Plato"){
                                    binding.iconoOn.visibility = View.INVISIBLE
                                    binding.iconoOff.visibility = View.INVISIBLE
                                    binding.iconoCambioPlato.visibility = View.VISIBLE
                                    binding.iconoComer.visibility = View.INVISIBLE
                                }
                                else if(lastPublishedMessage=="Dar de Comer"){
                                    binding.iconoOn.visibility = View.INVISIBLE
                                    binding.iconoOff.visibility = View.INVISIBLE
                                    binding.iconoCambioPlato.visibility = View.INVISIBLE
                                    binding.iconoComer.visibility = View.VISIBLE
                                }
                            }
                        }




                        mActivity?.runOnUiThread {
                            binding.textViewUmbralCejas?.text = "Umbral cejas: ${UsageInfoFragment.umbralCejas}% tamaño cara"
                            binding.textViewUmbralBoca?.text = "Umbral boca: ${UsageInfoFragment.umbralBoca}% tamaño cara"
                            binding.textViewUmbralOjoDer?.text = "Umbral ojo derecho: ${UsageInfoFragment.umbralOjoDer}% tamaño cara"
                            binding.textViewUmbralOjoIzq?.text = "Umbral ojo izquierda: ${UsageInfoFragment.umbralOjoIzq}% tamaño cara"
                        }


//                        //Para quitar los textos si así lo elige el usuario
//                        requireActivity().runOnUiThread{
//                            if(!UsageInfoFragment.visualizarTexto){
//                                binding.linearLayout1.visibility = View.GONE
//                                binding.textView.visibility = View.INVISIBLE
//                                binding.textView2.visibility = View.INVISIBLE
//                                binding.textView3.visibility = View.INVISIBLE
//                                binding.textView4.visibility = View.INVISIBLE
//                                binding.textView5.visibility = View.INVISIBLE
//                            } else {
//                                binding.linearLayout1.visibility = View.VISIBLE
//                                binding.textView.visibility = View.VISIBLE
//                                binding.textView2.visibility = View.VISIBLE
//                                binding.textView3.visibility = View.VISIBLE
//                                binding.textView4.visibility = View.VISIBLE
//                                binding.textView5.visibility = View.VISIBLE
//                            }
//                        }

                        //Para quitar las barras de progreso si así lo quiere el usuario
                        requireActivity().runOnUiThread {
                            if(!UsageInfoFragment.visualizarProgreso){
                                binding.linearLayout2.visibility = View.GONE
                                binding.textView6.visibility = View.INVISIBLE
                                binding.progressBarCejas.visibility = View.INVISIBLE
                                binding.textView7.visibility = View.INVISIBLE
                                binding.progressBarBoca.visibility = View.INVISIBLE
                                binding.textView8.visibility = View.INVISIBLE
                                binding.progressBarPosBoca.visibility = View.INVISIBLE
                                binding.textView9.visibility = View.INVISIBLE
                                binding.progressBarOjoDer.visibility = View.INVISIBLE
                                binding.textView10.visibility = View.INVISIBLE
                                binding.progressBarOjoIzq.visibility = View.INVISIBLE

                                binding.linearLayoutProgressBar.visibility = View.GONE
                                binding.linearLayoutProgressBar2.visibility = View.GONE
                                binding.linearLayoutProgressBar3.visibility = View.GONE
                                binding.linearLayoutProgressBar4.visibility = View.GONE
                                binding.linearLayoutProgressBar5.visibility = View.GONE
                                binding.textViewProgressBarCejas2.visibility = View.INVISIBLE
                                binding.progressBarCejas2.visibility = View.INVISIBLE
                                binding.textViewProgressBarBoca2.visibility = View.INVISIBLE
                                binding.progressBarBoca2.visibility = View.INVISIBLE
                                binding.textViewProgressBarPosBoca2.visibility = View.INVISIBLE
                                binding.progressBarPosBoca2.visibility = View.INVISIBLE
                                binding.textViewProgressBarOjoDer2.visibility = View.INVISIBLE
                                binding.progressBarOjoDer2.visibility = View.INVISIBLE
                                binding.textViewProgressBarOjoIzq2.visibility = View.INVISIBLE
                                binding.progressBarOjoIzq2.visibility = View.INVISIBLE
                            }
                        }

                        //Para poner las barras de progreso en la cara
                        requireActivity().runOnUiThread {
                            if(UsageInfoFragment.progressBarsEnCara && UsageInfoFragment.visualizarProgreso){
                                binding.linearLayout2.visibility = View.GONE
                                binding.textView6.visibility = View.INVISIBLE
                                binding.progressBarCejas.visibility = View.INVISIBLE
                                binding.textView7.visibility = View.INVISIBLE
                                binding.progressBarBoca.visibility = View.INVISIBLE
                                binding.textView8.visibility = View.INVISIBLE
                                binding.progressBarPosBoca.visibility = View.INVISIBLE
                                binding.textView9.visibility = View.INVISIBLE
                                binding.progressBarOjoDer.visibility = View.INVISIBLE
                                binding.textView10.visibility = View.INVISIBLE
                                binding.progressBarOjoIzq.visibility = View.INVISIBLE

                                binding.linearLayoutProgressBar.visibility = View.VISIBLE
                                binding.linearLayoutProgressBar2.visibility = View.VISIBLE
                                binding.linearLayoutProgressBar3.visibility = View.VISIBLE
                                binding.linearLayoutProgressBar4.visibility = View.VISIBLE
                                binding.linearLayoutProgressBar5.visibility = View.VISIBLE
                                binding.textViewProgressBarCejas2.visibility = View.VISIBLE
                                binding.progressBarCejas2.visibility = View.VISIBLE
                                binding.textViewProgressBarBoca2.visibility = View.VISIBLE
                                binding.progressBarBoca2.visibility = View.VISIBLE
                                binding.textViewProgressBarPosBoca2.visibility = View.VISIBLE
                                binding.progressBarPosBoca2.visibility = View.VISIBLE
                                binding.textViewProgressBarOjoDer2.visibility = View.VISIBLE
                                binding.progressBarOjoDer2.visibility = View.VISIBLE
                                binding.textViewProgressBarOjoIzq2.visibility = View.VISIBLE
                                binding.progressBarOjoIzq2.visibility = View.VISIBLE
                            }
                            else if(!UsageInfoFragment.progressBarsEnCara && UsageInfoFragment.visualizarProgreso){
                                binding.linearLayout2.visibility = View.VISIBLE
                                binding.textView6.visibility = View.VISIBLE
                                binding.progressBarCejas.visibility = View.VISIBLE
                                binding.textView7.visibility = View.VISIBLE
                                binding.progressBarBoca.visibility = View.VISIBLE
                                binding.textView8.visibility = View.VISIBLE
                                binding.progressBarPosBoca.visibility = View.VISIBLE
                                binding.textView9.visibility = View.VISIBLE
                                binding.progressBarOjoDer.visibility = View.VISIBLE
                                binding.textView10.visibility = View.VISIBLE
                                binding.progressBarOjoIzq.visibility = View.VISIBLE

                                binding.linearLayoutProgressBar.visibility = View.GONE
                                binding.linearLayoutProgressBar2.visibility = View.GONE
                                binding.linearLayoutProgressBar3.visibility = View.GONE
                                binding.linearLayoutProgressBar4.visibility = View.GONE
                                binding.linearLayoutProgressBar5.visibility = View.GONE
                                binding.textViewProgressBarCejas2.visibility = View.GONE
                                binding.progressBarCejas2.visibility = View.INVISIBLE
                                binding.textViewProgressBarBoca2.visibility = View.INVISIBLE
                                binding.progressBarBoca2.visibility = View.INVISIBLE
                                binding.textViewProgressBarPosBoca2.visibility = View.INVISIBLE
                                binding.progressBarPosBoca2.visibility = View.INVISIBLE
                                binding.textViewProgressBarOjoDer2.visibility = View.INVISIBLE
                                binding.progressBarOjoDer2.visibility = View.INVISIBLE
                                binding.textViewProgressBarOjoIzq2.visibility = View.INVISIBLE
                                binding.progressBarOjoIzq2.visibility = View.INVISIBLE
                            }
                        }

                        //Para conocer dimensiones de la pantalla(se usan factorX y factorY para dibujar)
                        val inputImage = InputImage.fromMediaImage(image.image!!, image.imageInfo.rotationDegrees)
                        val previewView = binding.previewView
                        val context = requireContext()
                        val displayMetrics = context.resources.displayMetrics
                        val width = displayMetrics.widthPixels
                        val height = displayMetrics.heightPixels

                        val imageWidth = inputImage.width
                        val imageHeight = inputImage.height
                        val factorX = width / imageWidth.toFloat()
                        val factorY = height / imageHeight.toFloat()

                        //Esta línea de código determina si el dispositivo es Tablet o Móvil (fundamental para el dibujado correcto y el layout)
                        val isTablet = context.resources.configuration.screenLayout and Configuration.SCREENLAYOUT_SIZE_MASK >= Configuration.SCREENLAYOUT_SIZE_LARGE

                        //Para que la interfaz original esté activada por defecto para la tablet
                        if (isTablet && UsageInfoFragment.visualizarInterfaz) {
                            UsageInfoFragment.visualizarInterfaz = true
                        } else { //Si es móvil o ya estaba desactivado
                            UsageInfoFragment.visualizarInterfaz = false
                        }

                        //Cálculos con los resultados del detector de malla facial
                        defaultDetector.process(inputImage)
                            .addOnSuccessListener { result ->
                                if(result.isEmpty()){
                                    Log.d("FaceMesh", "No se han detectado caras")

                                    //Para dejar de dibujar la cara
                                    mActivity?.runOnUiThread {
                                        val overlayView = binding.overlayView
                                        overlayView.background = null
                                    }

                                    //Para detener los gestos detectados en el último frame
                                    cejas = 1 // Valor por defecto para cejas
                                    boca = 1  // Valor por defecto para boca
                                    posBoca = 1 // Valor por defecto para posición de boca
                                    ojoDer = 1 // Valor por defecto para ojo derecho
                                    ojoIzq = 1 // Valor por defecto para ojo izquierdo

                                    //Para detener las animaciones de las barras de progreso
                                    detenerAnimacionesYReiniciarBarras()

                                    //Para evitar que se hagan acciones aún no habiendo cara
                                    detenerHandlers()

                                } else {
                                    for (faceMesh in result) {

                                        //Rectángulo alrededor de la cara (no lo uso)
                                        val bounds: Rect = faceMesh.boundingBox

                                        //Para vaciar los arrays de posiciones entre una ejecución y otra
                                        posicionesX.clear()
                                        posicionesY.clear()

                                        //En esta variable almaceno todos los puntos detectados
                                        val faceMeshpoints = faceMesh.allPoints
                                        //Para usarlo en el dibujado de las flechas
                                        val positionFlechas = faceMeshpoints[10].position
                                        val positionFlechas2 = faceMeshpoints[152].position
                                        val dy = positionFlechas.y-positionFlechas2.y
                                        val tamano = -dy*0.01


                                        //Llamada a la función que mete las coordenadas X e Y de cada punto en dos arrays distintos
                                        obtenerPosiciones(faceMeshpoints, posicionesX, posicionesY)

                                        //Llamadas a la función calcularDistancia, con el resultado determinaremos si se realizan, o no, gestos
                                        var distanciaExt = calcularDistancia(0, posicionesX, posicionesY)
                                        var distanciaCejas = calcularDistancia(1, posicionesX, posicionesY)
                                        var distanciaBoca = calcularDistancia(2, posicionesX, posicionesY)
                                        var distanciaBocaDer = calcularDistancia(3, posicionesX, posicionesY)
                                        var distanciaBocaIzq = calcularDistancia(4, posicionesX, posicionesY)
                                        var distanciaOjoDer = calcularDistancia(5, posicionesX, posicionesY)
                                        var distanciaOjoIzq = calcularDistancia(6, posicionesX, posicionesY)

//Para detener la ejecución cuando se duerme el usuario
                                        //Esta línea se ejecuta otra vez en determinarGesto, es solo para poder determinar lo de dormido antes de entrar en los gestos
                                        if((distanciaOjoDer/distanciaExt)<UsageInfoFragment.umbralOjoDer*0.01){
                                            ojoDer=1
                                        } else{
                                            ojoDer=0
                                        }
                                        if((distanciaOjoIzq/distanciaExt)<UsageInfoFragment.umbralOjoDer*0.01){
                                            ojoIzq=1
                                        } else {
                                            ojoIzq = 0
                                        }
                                        //Sabemos si el usuario tiene los ojos cerrados
                                        val runnableOjosCerrados = Runnable{
                                            dormido=true
                                        }
                                        if(ojoDer==1 && ojoIzq==1){
                                            handlerOjos.postDelayed(runnableOjosCerrados, 3000)
                                        } else {
                                            dormido = false
                                            handlerOjos.removeCallbacksAndMessages(null)
                                        }

                                        if(!dormido){
                                            //Usando los resultados de calcular las distancias, determinamos si se realizan los gestos faciales considerados
                                            cejas = determinarGesto(0, distanciaExt, distanciaCejas, 0f, 0f)
                                            boca = determinarGesto(1, distanciaExt, distanciaBoca, 0f, 0f)
                                            posBoca = determinarGesto(2,distanciaExt, distanciaBocaDer, distanciaBocaIzq, distanciaBoca)
                                            ojoDer = determinarGesto(3, distanciaExt, distanciaOjoDer, 0f, 0f)
                                            ojoIzq = determinarGesto(4, distanciaExt, distanciaOjoIzq, 0f, 0f)

                                            /*
                                            Se dibujan en pantalla los puntso faciales detectados (solo si el usuario quiere),
                                            cambiando su color en función de si se realiza algún gesto facial.

                                            Para ello, primero se comprueba si el usuario no ha desabilitado la función de dibujado
                                            Después, se crean el overlayView, el bitmap y el canvas, necesarios para dibujar los puntos
                                             */
                                            val overlayView = binding.overlayView
                                            val bitmap = Bitmap.createBitmap(overlayView.width, overlayView.height, Bitmap.Config.ARGB_8888)
                                            val canvas = Canvas(bitmap)

                                            //Esta condición era necesaria para evitar el error "fragment not attached to Activity"
                                            if(isAdded){
                                                mActivity?.runOnUiThread {
                                                    //Se elige el color de los puntos en función de si se realiza algún gesto, y después se dibuja
                                                    val paint = selectorColor(cejas, boca)

                                                    //Lo meto dentro de un if para que no lo haga siempre y quite eficiencia
                                                    if(UsageInfoFragment.visualizarTexto){
                                                        //Cejas y boca tienen la lógica contraria a ojos
                                                        //Abierto o subido = 0, cerrado o bajado =1
                                                        //lo que pasa es que el gesto de cejas y boca es abril y el de ojos es cerrar
                                                        if(cejas==0 && boca==0 && ojoDer==0 && ojoIzq==0)
                                                            textoAMostrar = "Cejas levantadas y boca abierta"
                                                        else if(cejas==0 && boca==0 && ojoDer==0 && ojoIzq==1)
                                                            textoAMostrar = "Cejas levantadas, boca abierta y ojo izquierdo cerrado"
                                                        else if(cejas==0 && boca==0 && ojoDer==1 && ojoIzq==0)
                                                            textoAMostrar = "Cejas levantadas, boca abierta y ojo derecho cerrado"
                                                        else if(cejas==0 && boca==0 && ojoDer==1 && ojoIzq==1)
                                                            textoAMostrar = "Cejas levantadas, boca abierta y ambos ojos cerrados"

                                                        else if(cejas==0 && boca==1 && ojoDer==0 && ojoIzq==0)
                                                            textoAMostrar = "Cejas levantadas"
                                                        else if(cejas==0 && boca==1 && ojoDer==0 && ojoIzq==1)
                                                            textoAMostrar = "Cejas levantadas y ojo izquierdo cerrado"
                                                        else if(cejas==0 && boca==1 && ojoDer==1 && ojoIzq==0)
                                                            textoAMostrar = "Cejas levantadas y ojo derecho cerrado"
                                                        else if(cejas==0 && boca==1 && ojoDer==1 && ojoIzq==1)
                                                            textoAMostrar = "Cejas levantadas y ambos ojos cerrados"

                                                        else if(cejas==1 && boca==0 && ojoDer==0 && ojoIzq==0)
                                                            textoAMostrar = "Boca abierta"
                                                        else if(cejas==1 && boca==0 && ojoDer==0 && ojoIzq==1)
                                                            textoAMostrar = "Boca abierta y ojo izquierdo cerrado"
                                                        else if(cejas==1 && boca==0 && ojoDer==1 && ojoIzq==0)
                                                            textoAMostrar = "Boca abierta y ojo derecho cerrado"
                                                        else if(cejas==1 && boca==0 && ojoDer==1 && ojoIzq==1)
                                                            textoAMostrar = "Boca abierta y ambos ojos cerrados"

                                                        //1 1 0 0 es ningún gesto
                                                        else if(cejas==1 && boca==1 && ojoDer==0 && ojoIzq==1)
                                                            textoAMostrar = "Ojo izquierdo cerrado"
                                                        else if(cejas==1 && boca==1 && ojoDer==1 && ojoIzq==0)
                                                            textoAMostrar = "Ojo derecho cerrado"
                                                        else if(cejas==1 && boca==1 && ojoDer==1 && ojoIzq==1)
                                                            textoAMostrar = "Ambos ojos cerrados"

                                                        else
                                                            textoAMostrar = "Ningún gesto detectado"
                                                    }


                                                    if(UsageInfoFragment.visualizarPuntos)
                                                        dibujaPuntos(faceMeshpoints, overlayView, bitmap, canvas, paint, isTablet, factorX, factorY, width, height)

                                                    if(UsageInfoFragment.visualizarTexto)
                                                        dibujaTextos(faceMeshpoints, overlayView, bitmap, canvas, paint, isTablet, factorX, factorY, width, height, textoAMostrar,tamano)

                                                    val paintCejas = selectorColorCejas(cejas)
                                                    dibujaPuntosCejas(faceMeshpoints,canvas, paintCejas, isTablet, factorX, factorY, width, height)

                                                    val paintBoca = selectorColorBoca(boca,posBoca)
                                                    dibujaPuntosBoca(faceMeshpoints, canvas, paintBoca, isTablet, factorX, factorY, width, height)

                                                    val paintOjoDer = selectorColorOjoDer(ojoDer)
                                                    dibujaPuntosOjoDer(faceMeshpoints, canvas, paintOjoDer, isTablet, factorX, factorY, width, height)

                                                    val paintOjoIzq = selectorColorOjoIzq(ojoIzq)
                                                    dibujaPuntosOjoIzq(faceMeshpoints, canvas, paintOjoIzq, isTablet, factorX, factorY, width, height)

                                                    if(UsageInfoFragment.visualizarFlechas){
                                                        dibujaFlechaCejas(faceMeshpoints, canvas, paintCejas, isTablet, factorX, factorY, width, height, tamano)
                                                        //dibujaFlechaCejasPrueba(faceMeshpoints, canvas, cejas, isTablet, factorX, factorY, width, height, tamano)
                                                        dibujaFlechaBoca(faceMeshpoints, canvas, paintBoca, isTablet, factorX, factorY, width, height, tamano)
                                                        //dibujaFlechaBocaPrueba(faceMeshpoints, canvas, boca, isTablet, factorX, factorY, width, height, tamano)
                                                        dibujaFlechaOjoDer(faceMeshpoints, canvas, paintOjoDer, isTablet, factorX, factorY, width, height, tamano)
                                                        dibujaFlechaOjoIzq(faceMeshpoints, canvas, paintOjoIzq, isTablet, factorX, factorY, width, height, tamano)
                                                    }

                                                    if(UsageInfoFragment.progressBarsEnCara){
                                                        actualizarPosicionProgressBars(posicionesX, posicionesY, factorX, width, height, isTablet)
                                                    }

                                                    //Añadido aquí para intentar haber guardado el último topic
                                                    preferencesTopicoActual.edit().putInt("topicoActual",topicoActual).apply()

                                                    overlayView.background = BitmapDrawable(resources,bitmap)
                                                }
                                            }
                                        } else {
                                            //Dormido, debería cortar la ejecución de los gestos
                                            detenerHandlers()
                                            detenerAnimacionesYReiniciarBarras()

                                            //Era un intento de modo oscuro pero no se cambiar el toolbar
//                                            binding.linearLayoutPrincipal.setBackgroundColor(Color.BLACK)
//                                            binding.textViewConect.setTextColor(Color.WHITE)
//                                            binding.textViewGestos.setTextColor(Color.WHITE)
//                                            binding.textViewUltimoMensaje.setTextColor(Color.WHITE)
//                                            binding.textViewTopicoActual.setTextColor(Color.WHITE)
//                                            binding.linearLayoutUmbrales?.setBackgroundColor(Color.BLACK)
//                                            binding.textViewUmbralCejas?.setTextColor(Color.WHITE)
//                                            binding.textViewUmbralBoca?.setTextColor(Color.WHITE)
//                                            binding.textViewUmbralOjoDer?.setTextColor(Color.WHITE)
//                                            binding.textViewUmbralOjoIzq?.setTextColor(Color.WHITE)
                                        }
                                        ////////////////////////////////////////////////
                                    }
                                }
                                image.close()
                            }
                            .addOnFailureListener {
                                println(it.message)
                            }
                    }
                })}
    }









    //Función para publicar mensajes en el servidor MQTT
    fun publish(topic: String, data: String) {

//
        var topicToPublish = topic

//        when(topicoActual){
//            1-> {
//                topicToPublish = MQTTConfigFragment.publishTopic1
//            }
//            2->{
//                topicToPublish = MQTTConfigFragment.publishTopic2
//            }
//            3->{
//                topicToPublish = MQTTConfigFragment.publishTopic3
//            }
//            4->{
//                topicToPublish = MQTTConfigFragment.publishTopic4
//            }
//            5->{
//                topicToPublish = MQTTConfigFragment.publishTopic5
//            }
//        }
//


        val encodedPayload : ByteArray
        try {
            encodedPayload = data.toByteArray(charset("UTF-8"))
            val message = MqttMessage(encodedPayload)
            message.qos = 0
            message.isRetained = false
            MQTTConfigFragment.mqttClient!!.publish(topicToPublish, message, null, object: IMqttActionListener {
                override fun onSuccess(asyncActionToken: IMqttToken?) {
                    Log.d(MQTTConfigFragment.TAG, "Publicado correctamente en $topicToPublish")
                }
                override fun onFailure(asyncActionToken: IMqttToken?, exception: Throwable?) {
                    Log.e(MQTTConfigFragment.TAG, "Error en la publicación")
                }
            })
        } catch (e: Exception) {
            e.printStackTrace()
        } catch (e: MqttException) {
            e.printStackTrace()
        }
    }



    //Para enviar mensajes por Bluetooth
    private fun sendCommand(input: String) {
        if (BTConfigFragment.m_bluetoothSocket != null) {
            try{
                BTConfigFragment.m_bluetoothSocket!!.outputStream.write(input.toByteArray())
            } catch(e: IOException) {
                e.printStackTrace()
            }
        }
    }




    //Para detener las animaciones de las barras de progreso
    fun detenerAnimacionesYReiniciarBarras() {
        binding.progressBarCejas.progress = 0
        binding.progressBarCejas.clearAnimation()
        binding.progressBarCejas2.progress = 0
        binding.progressBarCejas2.clearAnimation()
        binding.progressBarBoca.progress = 0
        binding.progressBarBoca.clearAnimation()
        binding.progressBarBoca2.progress = 0
        binding.progressBarBoca2.clearAnimation()
        binding.progressBarPosBoca.progress = 0
        binding.progressBarPosBoca.clearAnimation()
        binding.progressBarPosBoca2.progress = 0
        binding.progressBarPosBoca2.clearAnimation()
        binding.progressBarOjoDer.progress = 0
        binding.progressBarOjoDer.clearAnimation()
        binding.progressBarOjoDer2.progress = 0
        binding.progressBarOjoDer2.clearAnimation()
        binding.progressBarOjoIzq.progress = 0
        binding.progressBarOjoIzq.clearAnimation()
        binding.progressBarOjoIzq2.progress = 0
        binding.progressBarOjoIzq2.clearAnimation()
    }



    // Función para detener handlers
    fun detenerHandlers() {
        handlerCejas.removeCallbacksAndMessages(null)
        handlerBoca.removeCallbacksAndMessages(null)
        handlerPosBoca.removeCallbacksAndMessages(null)
        handlerOjoDer.removeCallbacksAndMessages(null)
        handlerOjoIzq.removeCallbacksAndMessages(null)
    }











    //Función que introduce las coordenadas de los puntos faciales a dos arrays
    fun obtenerPosiciones(faceMeshpoints: MutableList<FaceMeshPoint>, posicionesX: MutableList<Float>, posicionesY: MutableList<Float>){
        for ((index,faceMeshpoint) in faceMeshpoints.withIndex()) {
            val position = faceMeshpoint.position
            posicionesX.add(position.x)
            posicionesY.add(position.y)
        }
    }


    //Función que usa las coordenadas X e Y de ciertos puntos faciales para saber distancias entre cejas y ojos, por ejemplo
    fun calcularDistancia(operacion: Int, posicionesX: MutableList<Float>, posicionesY: MutableList<Float>): Float{

        var dx: Float
        var dy: Float

        when(operacion){
            0-> {
                dx = posicionesX[152]-posicionesX[10]
                dy = posicionesY[152]-posicionesY[10]
            }
            1-> {
                dx = posicionesX[145]-posicionesX[52]
                dy = posicionesY[145]-posicionesY[52]
            }
            2-> {
                dx = posicionesX[14]-posicionesX[13]
                dy = posicionesY[14]-posicionesY[13]
            }
            3-> {
                dx = posicionesX[146]-posicionesX[132]
                dy = posicionesY[146]-posicionesY[132]
            }
            4-> {
                dx = posicionesX[375]-posicionesX[361]
                dy = posicionesY[375]-posicionesY[361]
            }
            5->{
                dx = posicionesX[145]-posicionesX[159]
                dy = posicionesY[145]-posicionesY[159]
            }
            else-> {
                dx = posicionesX[374]-posicionesX[386]
                dy = posicionesY[374]-posicionesY[386]
            }
        }

        var distanciaExt = sqrt((dx * dx) + (dy * dy))

        return distanciaExt
    }


    /*Variables para evitar enviar 2000 mensajes al servidor MQTT por segundo
    Se usa un if para comprobar si el mensaje a enviar coincide con el último enviado,
    y si lo hace, no se envía*/
    //var lastPublishedMessageCejas: String? = null
    //var lastPublishedMessageBoca: String? = null //Comentado porque no lo uso(de momento al menos)
    var lastPublishedMessagePosBoca: String? = null
    //var lastPublishedMessageOjoDer: String? = null
    //var lastPublishedMessageOjoIzq: String? = null

    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
    @SuppressLint("SuspiciousIndentation")
    fun determinarGesto(gesto: Int, distanciaExt: Float, distanciaGesto: Float, distanciaGesto2: Float, distanciaGesto3: Float): Int{

        //Variable que uso como booleana (salvo en PosBoca) para saber si se realiza el gesto
        var seRealiza: Int

        //Mensaje de formato JSON que se envía al servidor MQTT para controlar la lámpara
        val cejasObject= JSONObject()

        //Se establece un máximo a las barras de progreso para que se vea correctamente su rellenado
        binding.progressBarCejas.max = 500
        binding.progressBarBoca.max = 500
        binding.progressBarPosBoca.max = 500
        binding.progressBarOjoDer.max = 500
        binding.progressBarOjoIzq.max = 500

        binding.progressBarCejas2.max = 500
        binding.progressBarBoca2.max = 500
        binding.progressBarPosBoca2.max = 500
        binding.progressBarOjoDer2.max = 500
        binding.progressBarOjoIzq2.max = 500

        //Uso una estructura when para llamar a la función usando un número que determina el gesto a analizar
        //0=cejas,1=boca,2=posBoca,3=ojoDer,4(else)=ojoIzq
        when(gesto){
            //Cejas
            0->{
                //Uso un runnable para meterlo directamente al handler dentro del if
                /*
                En cada runnable, se cambia el texto encima de la barra de progreso,
                se pone ON o OFF al JSON para controlar la domótica
                Se comprueba el tiempo entre encendido o apagado para no parpadear la bombilla mucho
                 */
                val runnableCejasSubidas = Runnable {
                    binding.textView6.text = "Cejas levantadas durante 5 segundos consecutivos"
                    binding.textViewProgressBarCejas2.text = "Cejas levantadas durante 5 segundos consecutivos"
                    //transcurridoDesdeEncendido = System.currentTimeMillis() - ultimoEncendido
                    //transcurridoDesdeApagado = System.currentTimeMillis() - ultimoApagado
                    transcurridoDesdeEnviado = System.currentTimeMillis() - ultimoEnviado
                    transcurridoDesdeConexion = System.currentTimeMillis() - ultimaConexion
                    transcurridoDesdeBT = System.currentTimeMillis() - ultimoBT
                    transcurridoDesdeCambio = System.currentTimeMillis() - ultimoCambio
                    if((MQTTConfigFragment.conectado || BTConfigFragment.m_isConnected) && UsageInfoFragment.cejasEncender){
                        if(MQTTConfigFragment.conectado && transcurridoDesdeEnviado >= 2000){
                            cejasObject.put("state","ON")
                            publish(MQTTConfigFragment.publishTopic, cejasObject.toString())
                            lastPublishedMessage = "EnviadoON"
                            ultimoEnviado = System.currentTimeMillis()
                        }
                        if(BTConfigFragment.m_isConnected && transcurridoDesdeBT >= 5000){
                            sendCommand("A")
                            lastPublishedMessage = "Cambio Plato"
                            ultimoBT = System.currentTimeMillis()
                        }
                        //ultimoEncendido = System.currentTimeMillis()
                    }
                    else if((MQTTConfigFragment.conectado || BTConfigFragment.m_isConnected) && UsageInfoFragment.cejasApagar){
                        if(MQTTConfigFragment.conectado && transcurridoDesdeEnviado >= 2000){
                            cejasObject.put("state","OFF")
                            publish(MQTTConfigFragment.publishTopic, cejasObject.toString())
                            lastPublishedMessage = "EnviadoOFF"
                            ultimoEnviado = System.currentTimeMillis()
                        }
                        if(BTConfigFragment.m_isConnected && transcurridoDesdeBT >= 5000){
                            sendCommand("B")
                            lastPublishedMessage = "Dar de Comer"
                            ultimoBT = System.currentTimeMillis()
                        }
                        //ultimoApagado = System.currentTimeMillis()
                    }
                        //Intento de conectar MQTT con gestos
/////////////////////////////////////////////////////////////
                    else if(MQTTConfigFragment.conectado && transcurridoDesdeConexion >= 5000 && UsageInfoFragment.cejasConexion){
                        MQTTConfigFragment().desconexionMQTT()
                        Toast.makeText(requireContext(),"Desconectado de MQTT",Toast.LENGTH_SHORT).show()
                        ultimaConexion = System.currentTimeMillis()
                    }
                    else if(BTConfigFragment.m_isConnected && transcurridoDesdeConexion >=5000 && UsageInfoFragment.cejasConexion){
                        BTConfigFragment().disconnect()
                        Toast.makeText(requireContext(),"Desconectado de Bluetooth",Toast.LENGTH_SHORT).show()
                        ultimaConexion = System.currentTimeMillis()
                    }
                    else if(!MQTTConfigFragment.conectado && !BTConfigFragment.m_isConnected && transcurridoDesdeConexion >= 5000 && UsageInfoFragment.cejasConexion){
                        Log.d("Intento MQTT", "Entró al if de conexión con gesto")
                        if (!MQTToBT) { //Si lo último conectado fue BT
                            MQTToBT = true //Porque se intentó acceder a MQTT, aunque no se haya metido direcciones se pone para que el siguiente sea Bluetooth

                            //Para ver si puedo conectarme sin entrar en MQTTConfigFragment
                            MQTTConfigFragment.direccion = ConexionBBDDSQLite(requireContext()).getDnsById(1).toString()
                            MQTTConfigFragment.puerto = ConexionBBDDSQLite(requireContext()).getPuertoById(1).toString()
                            MQTTConfigFragment.serverUri = "tcp://${MQTTConfigFragment.direccion}:${MQTTConfigFragment.puerto}"
                            MQTTConfigFragment.usuarioMQTT = ConexionBBDDSQLite(requireContext()).getUserNameById(1).toString()
                            MQTTConfigFragment.contrasenaMQTT = ConexionBBDDSQLite(requireContext()).getContrasenaById(1).toString()
                            Log.d("IntentoConexionMQTT", "URI:${MQTTConfigFragment.serverUri}")
                            Toast.makeText(requireContext(),"Intentando conexión MQTT",Toast.LENGTH_SHORT).show()

                            MQTTConfigFragment().conexionMQTT(requireContext(),MQTTConfigFragment.serverUri) { exitosa ->
                                if(exitosa){
                                    val handler = Handler(Looper.getMainLooper())
                                    handler.post {
                                        Toast.makeText(requireContext(), "Conexión MQTT exitosa", Toast.LENGTH_LONG).show()
                                    }
                                } else {
                                    val handler = Handler(Looper.getMainLooper())
                                    handler.post {
                                        Toast.makeText(requireContext(), "Conexión MQTT fallida, vuelve a intentarlo", Toast.LENGTH_LONG).show()
                                    }
                                }
                            }
                        } else {
                            MQTToBT = false
                            val BTpreferences = requireContext().getSharedPreferences("BTPreferences", Context.MODE_PRIVATE)
                            BTConfigFragment.m_address = BTpreferences.getString("direccionBTGuardada", "").toString()
                            Log.d("Bluetooth", "DireccionBT obtenida del preferences: ${BTConfigFragment.m_address}")
                            Toast.makeText(requireContext(),"Intentando conexión Bluetooth",Toast.LENGTH_SHORT).show()
                            if(BTConfigFragment.m_address!=""){
                                var mBtAdapter = (requireContext().getSystemService(Context.BLUETOOTH_SERVICE) as BluetoothManager).adapter
                                val device: BluetoothDevice = mBtAdapter.getRemoteDevice(BTConfigFragment.m_address)
                                if (ActivityCompat.checkSelfPermission(requireContext(), Manifest.permission.BLUETOOTH_CONNECT) != PackageManager.PERMISSION_GRANTED) {
                                    ActivityCompat.requestPermissions(
                                        requireActivity(), arrayOf(
                                            Manifest.permission.BLUETOOTH,
                                            Manifest.permission.BLUETOOTH_CONNECT,
                                            Manifest.permission.BLUETOOTH_ADMIN,
                                            Manifest.permission.BLUETOOTH_SCAN,
                                            Manifest.permission.BLUETOOTH_ADVERTISE,
                                            Manifest.permission.BLUETOOTH_PRIVILEGED,
                                            Manifest.permission.MANAGE_DEVICE_POLICY_BLUETOOTH,
                                        ), 1)
                                }
                                connectToDevice(device)
                            } else {
                                Toast.makeText(requireContext(),"No hay dirección de dispositivo BT registrada",Toast.LENGTH_SHORT).show()
                            }
                        }
                        ultimaConexion = System.currentTimeMillis()
                    }
                    else if(transcurridoDesdeCambio>=5000 && UsageInfoFragment.cejasCambiar){
                        if(topicoActual<MQTTConfigFragment.numeroTopics){
                            topicoActual=topicoActual+1
                        } else {
                            topicoActual=1
                        }
                        ultimoCambio = System.currentTimeMillis()
                    }
////////////////////////////////////////
                }
                if((distanciaGesto/distanciaExt)>(UsageInfoFragment.umbralCejas*0.01)){
                //if((distanciaGesto/distanciaExt)>0.14){
                    seRealiza = 0
                    if(seRealiza==0){
                        handlerCejas.postDelayed(runnableCejasSubidas, 3000)
                    } else {
                        handlerCejas.removeCallbacks(runnableCejasSubidas)
                    }
//                    mActivity?.runOnUiThread {
//                        binding.textView.text = "Cejas levantadas"
//                    }
                } else{
                    seRealiza = 1
                    handlerCejas.removeCallbacksAndMessages(null)
                    binding.progressBarCejas.progress = 0
                    ObjectAnimator.ofInt(binding.progressBarCejas,"progress", 500)
                        .setDuration(3000)
                        .start()
                    binding.progressBarCejas2.progress = 0
                    ObjectAnimator.ofInt(binding.progressBarCejas2,"progress", 500)
                        .setDuration(3000)
                        .start()
                    binding.textView6.text = "Gesto cejas:"
                    binding.textViewProgressBarCejas2.text = "Gesto cejas:"
//                    mActivity?.runOnUiThread {
//                        binding.textView.text = "Cejas bajadas"
//                    }
                }
            }

            1->{
                val runnableBocaAbierta = Runnable {
                    binding.textView7.text = "Boca abierta durante 5 segundos consecutivos"
                    binding.textViewProgressBarBoca2.text = "Boca abierta durante 5 segundos consecutivos"
                    //transcurridoDesdeEncendido = System.currentTimeMillis() - ultimoEncendido
                    //transcurridoDesdeApagado = System.currentTimeMillis() - ultimoApagado
                    transcurridoDesdeEnviado = System.currentTimeMillis() - ultimoEnviado
                    transcurridoDesdeBT = System.currentTimeMillis() - ultimoBT
                    transcurridoDesdeConexion = System.currentTimeMillis() - ultimaConexion
                    transcurridoDesdeCambio = System.currentTimeMillis() - ultimoCambio
                    if((BTConfigFragment.m_isConnected || MQTTConfigFragment.conectado) && UsageInfoFragment.bocaEncender){
                        if(MQTTConfigFragment.conectado && transcurridoDesdeEnviado >= 2000){
                            cejasObject.put("state","ON")
                            publish(MQTTConfigFragment.publishTopic, cejasObject.toString())
                            lastPublishedMessage = "EnviadoON"
                            ultimoEnviado = System.currentTimeMillis()
                        }
                        if(BTConfigFragment.m_isConnected && transcurridoDesdeBT >= 5000){
                            sendCommand("A")
                            lastPublishedMessage = "Cambio Plato"
                            ultimoBT = System.currentTimeMillis()
                        }
                        //ultimoApagado = System.currentTimeMillis()
                    }
                    else if((MQTTConfigFragment.conectado || BTConfigFragment.m_isConnected) && UsageInfoFragment.bocaApagar){
                        if(MQTTConfigFragment.conectado && transcurridoDesdeEnviado >= 2000){
                            cejasObject.put("state","OFF")
                            publish(MQTTConfigFragment.publishTopic, cejasObject.toString())
                            lastPublishedMessage = "EnviadoOFF"
                            ultimoEnviado = System.currentTimeMillis()
                        }
                        if(BTConfigFragment.m_isConnected && transcurridoDesdeBT >= 5000){
                            sendCommand("B")
                            lastPublishedMessage = "Dar de Comer"
                            ultimoBT = System.currentTimeMillis()
                        }
                        //ultimoApagado = System.currentTimeMillis()
                    }
                    else if(MQTTConfigFragment.conectado && transcurridoDesdeConexion >= 5000 && UsageInfoFragment.bocaConexion){
                        MQTTConfigFragment().desconexionMQTT()
                        Toast.makeText(requireContext(),"Desconectado",Toast.LENGTH_SHORT).show()
                        ultimaConexion = System.currentTimeMillis()
                    }
                    else if(BTConfigFragment.m_isConnected && transcurridoDesdeConexion >=5000 && UsageInfoFragment.bocaConexion){
                        BTConfigFragment().disconnect()
                        Toast.makeText(requireContext(),"Desconectado de Bluetooth",Toast.LENGTH_SHORT).show()
                        ultimaConexion = System.currentTimeMillis()
                    }
                    else if(!MQTTConfigFragment.conectado && !BTConfigFragment.m_isConnected && transcurridoDesdeConexion >= 5000 && UsageInfoFragment.bocaConexion){

                        if (!MQTToBT) { //Si lo último conectado fue BT
                            MQTToBT = true //Porque se intentó acceder a MQTT, aunque no se haya metido direcciones se pone para que el siguiente sea Bluetooth
                            //Para ver si puedo conectarme a la primera
                            MQTTConfigFragment.direccion = ConexionBBDDSQLite(requireContext()).getDnsById(1).toString()
                            MQTTConfigFragment.puerto = ConexionBBDDSQLite(requireContext()).getPuertoById(1).toString()
                            MQTTConfigFragment.serverUri = "tcp://${MQTTConfigFragment.direccion}:${MQTTConfigFragment.puerto}"
                            MQTTConfigFragment.usuarioMQTT = ConexionBBDDSQLite(requireContext()).getUserNameById(1).toString()
                            MQTTConfigFragment.contrasenaMQTT = ConexionBBDDSQLite(requireContext()).getContrasenaById(1).toString()
                            Log.d("IntentoConexionALaPrimera", "URI:${MQTTConfigFragment.serverUri}")
                            Toast.makeText(requireContext(),"Intentando conexión MQTT",Toast.LENGTH_SHORT).show()

                            MQTTConfigFragment().conexionMQTT(requireContext(),MQTTConfigFragment.serverUri) { exitosa ->
                                if(exitosa){
                                    requireActivity().runOnUiThread {
                                        Toast.makeText(requireContext(), "Conexión MQTT exitosa", Toast.LENGTH_LONG).show()
                                    }
                                } else {
                                    val handler = Handler(Looper.getMainLooper())
                                    handler.post {
                                        Toast.makeText(requireContext(), "Conexión MQTT fallida, vuelve a intentarlo", Toast.LENGTH_LONG).show()
                                    }
                                }
                            }
                        } else {
                            MQTToBT = false
                            val BTpreferences = requireContext().getSharedPreferences("BTPreferences", Context.MODE_PRIVATE)
                            BTConfigFragment.m_address = BTpreferences.getString("direccionBTGuardada", "").toString()
                            Log.d("Bluetooth", "DireccionBT obtenida del preferences: ${BTConfigFragment.m_address}")
                            Toast.makeText(requireContext(),"Intentando conexión Bluetooth",Toast.LENGTH_SHORT).show()
                            if(BTConfigFragment.m_address!=""){
                                var mBtAdapter = (requireContext().getSystemService(Context.BLUETOOTH_SERVICE) as BluetoothManager).adapter
                                val device: BluetoothDevice = mBtAdapter.getRemoteDevice(BTConfigFragment.m_address)
                                if (ActivityCompat.checkSelfPermission(requireContext(), Manifest.permission.BLUETOOTH_CONNECT) != PackageManager.PERMISSION_GRANTED) {
                                    ActivityCompat.requestPermissions(
                                        requireActivity(), arrayOf(
                                            Manifest.permission.BLUETOOTH,
                                            Manifest.permission.BLUETOOTH_CONNECT,
                                            Manifest.permission.BLUETOOTH_ADMIN,
                                            Manifest.permission.BLUETOOTH_SCAN,
                                            Manifest.permission.BLUETOOTH_ADVERTISE,
                                            Manifest.permission.BLUETOOTH_PRIVILEGED,
                                            Manifest.permission.MANAGE_DEVICE_POLICY_BLUETOOTH,
                                        ), 1)
                                }
                                connectToDevice(device)
                            } else {
                                Toast.makeText(requireContext(),"No hay dirección de dispositivo BT registrada",Toast.LENGTH_SHORT).show()
                            }
                        }
                        ultimaConexion = System.currentTimeMillis()
                    }
                    else if(transcurridoDesdeCambio>=5000 && UsageInfoFragment.bocaCambiar){
                        if(topicoActual<MQTTConfigFragment.numeroTopics){
                            topicoActual=topicoActual+1
                        } else {
                            topicoActual=1
                        }
                        ultimoCambio = System.currentTimeMillis()
                    }
                }
                if((distanciaGesto/distanciaExt)>(UsageInfoFragment.umbralBoca*0.01)){
                //if((distanciaGesto/distanciaExt)>0.08){
                    seRealiza = 0
                    if(seRealiza==0){
                        handlerBoca.postDelayed(runnableBocaAbierta, 5000)
                    } else {
                        handlerBoca.removeCallbacks(runnableBocaAbierta)
                    }
//                    mActivity?.runOnUiThread {
//                        binding.textView2.text = "Boca abierta"
//                    }
                } else{
                    seRealiza = 1
                    handlerBoca.removeCallbacksAndMessages(null)
                    binding.progressBarBoca.progress = 0
                    ObjectAnimator.ofInt(binding.progressBarBoca,"progress", 500)
                        .setDuration(5000)
                        .start()
                    binding.progressBarBoca2.progress = 0
                    ObjectAnimator.ofInt(binding.progressBarBoca2,"progress", 500)
                        .setDuration(5000)
                        .start()
                    binding.textView7.text = "Gesto boca:"
                    binding.textViewProgressBarBoca2.text = "Gesto boca:"
//                    mActivity?.runOnUiThread {
//                        binding.textView2.text = "Boca cerrada"
//                    }
                }
            }

            2->{
                val runnableBocaDerecha = Runnable {
                    binding.textView8.text = "Boca a la derecha 5 segundos consecutivos"
                    binding.textViewProgressBarPosBoca2.text = "Boca a la derecha 5 segundos consecutivos"
                    //cejasObject.put("state","OFF")
                    //val transcurridoDesdeApagado = System.currentTimeMillis() - ultimoApagado
                    if(MQTTConfigFragment.conectado && lastPublishedMessagePosBoca!="Enviado1"){
                        //publish(MQTTConfigFragment.publishTopic3, "A la derecha")
                        //ultimoApagado = System.currentTimeMillis()
                        lastPublishedMessagePosBoca = "Enviado1"
                    }
                }
                val runnableBocaIzquierda = Runnable {
                    binding.textView8.text = "Boca a la izquierda 5 segundos consecutivos"
                    binding.textViewProgressBarPosBoca2.text = "Boca a la izquierda 5 segundos consecutivos"
                    //cejasObject.put("state","OFF")
                    //val transcurridoDesdeApagado = System.currentTimeMillis() - ultimoApagado
                    if(MQTTConfigFragment.conectado && lastPublishedMessagePosBoca!="Enviado2"){
                        //publish(MQTTConfigFragment.publishTopic3, "A la izquierda")
                        //ultimoApagado = System.currentTimeMillis()
                        lastPublishedMessagePosBoca = "Enviado2"
                    }
                }
                if((distanciaGesto/distanciaExt)<0.28 && (distanciaGesto2/distanciaExt>0.31)){
                    seRealiza = 1
                    handlerPosBoca.postDelayed(runnableBocaDerecha, 5000)
//                    mActivity?.runOnUiThread {
//                        binding.textView3.text = "Boca a la derecha"
//                    }
                }
                else if((distanciaGesto/distanciaExt)>0.31 && (distanciaGesto2/distanciaExt<0.28)){
                    seRealiza = 2
                    handlerPosBoca.postDelayed(runnableBocaIzquierda, 5000)
//                    mActivity?.runOnUiThread {
//                        binding.textView3.text = "Boca a la izquierda"
//                    }
                }
                else if((distanciaGesto/distanciaExt)<0.23 && (distanciaGesto2/distanciaExt)<0.23 && (distanciaGesto3/distanciaExt)<0.05) {
                    seRealiza = 3
                    handlerPosBoca.postDelayed({
                        binding.textView8.text = "Boca centrada y sonriendo sin dientes durante 5 segundos consecutivos"
                        binding.textViewProgressBarPosBoca2.text = "Boca centrada y sonriendo sin dientes durante 5 segundos consecutivos"
                        if(MQTTConfigFragment.conectado && lastPublishedMessagePosBoca!="Enviado3")
                            //publish(MQTTConfigFragment.publishTopic3, "Sonriendo sin dientes")
                        lastPublishedMessagePosBoca="Enviado3"
                    }, 5000)
//                    mActivity?.runOnUiThread {
//                        binding.textView3.text = "Boca centrada y sonriendo sin dientes"
//                    }
                }
                else if((distanciaGesto/distanciaExt)<0.24 && (distanciaGesto2/distanciaExt)<0.24 && (distanciaGesto3/distanciaExt)>=0.05){
                    seRealiza = 4
                    handlerPosBoca.postDelayed({
                        binding.textView8.text = "Boca centrada y sonriendo con dientes durante 5 segundos consecutivos"
                        binding.textViewProgressBarPosBoca2.text = "Boca centrada y sonriendo con dientes durante 5 segundos consecutivos"
                        if(MQTTConfigFragment.conectado && lastPublishedMessagePosBoca!="Enviado4")
                            //publish(MQTTConfigFragment.publishTopic3, "Sonriendo con dientes")
                        lastPublishedMessagePosBoca = "Enviado4"
                    }, 5000)
//                    mActivity?.runOnUiThread {
//                        binding.textView3.text = "Boca centrada y sonriendo con dientes"
//                    }
                }
                else if((distanciaGesto/distanciaExt)>0.28 && (distanciaGesto2/distanciaExt)>0.28 && (distanciaGesto3/distanciaExt)<0.05){
                    seRealiza = 5
                    handlerPosBoca.postDelayed({
                        binding.textView8.text = "Boca hacia fuera durante 5 segundos consecutivos"
                        binding.textViewProgressBarPosBoca2.text = "Boca hacia fuera durante 5 segundos consecutivos"
                        if(MQTTConfigFragment.conectado && lastPublishedMessagePosBoca!="Enviado5")
                            //publish(MQTTConfigFragment.publishTopic3, "Hacia fuera")
                        lastPublishedMessagePosBoca = "Enviado5"
                    }, 5000)
//                    mActivity?.runOnUiThread {
//                        binding.textView3.text = "Boca hacia fuera"
//                    }
                }
                else if((distanciaGesto/distanciaExt)>0.28 && (distanciaGesto2/distanciaExt)>0.28 && (distanciaGesto3/distanciaExt)>=0.05){
                    seRealiza = 6
                    handlerPosBoca.postDelayed({
                        binding.textView8.text = "Boca de sorprendido durante 5 segundos consecutivos"
                        binding.textViewProgressBarPosBoca2.text = "Boca de sorprendido durante 5 segundos consecutivos"
                        if(MQTTConfigFragment.conectado && lastPublishedMessagePosBoca!="Enviado6")
                            //publish(MQTTConfigFragment.publishTopic3, "Sorprendido")
                        lastPublishedMessagePosBoca = "Enviado6"
                    }, 5000)
//                    mActivity?.runOnUiThread {
//                        binding.textView3.text = "Boca de sorprendido"
//                    }
                } else {
                    seRealiza = 0
                    handlerPosBoca.removeCallbacksAndMessages(null)
                    binding.textView8.text = "Posición boca:"
                    binding.textViewProgressBarPosBoca2.text = "Posición boca:"
                    binding.progressBarPosBoca.progress = 0
                    ObjectAnimator.ofInt(binding.progressBarPosBoca,"progress", 500)
                        .setDuration(5000)
                        .start()
                    binding.progressBarPosBoca2.progress = 0
                    ObjectAnimator.ofInt(binding.progressBarPosBoca2,"progress", 500)
                        .setDuration(5000)
                        .start()
                    if(MQTTConfigFragment.conectado && lastPublishedMessagePosBoca!="Enviado0")
                        //publish(MQTTConfigFragment.publishTopic3, "Centrada")
                    lastPublishedMessagePosBoca = "Enviado0"
//                    mActivity?.runOnUiThread {
//                        binding.textView3.text = "Boca centrada"
//                    }
                }
            }

            3->{
                val runnableOjoDerechoCerrado = Runnable {
                    binding.textView9.text = "Ojo derecho cerrado durante 5 segundos consecutivos"
                    binding.textViewProgressBarOjoDer2.text = "Ojo derecho cerrado durante 5 segundos consecutivos"
                    //transcurridoDesdeEncendido = System.currentTimeMillis() - ultimoEncendido
                    //transcurridoDesdeApagado = System.currentTimeMillis() - ultimoApagado
                    transcurridoDesdeEnviado = System.currentTimeMillis() - ultimoEnviado
                    transcurridoDesdeBT = System.currentTimeMillis() - ultimoBT
                    transcurridoDesdeConexion = System.currentTimeMillis() - ultimaConexion
                    transcurridoDesdeCambio = System.currentTimeMillis() - ultimoCambio
                    if((MQTTConfigFragment.conectado || BTConfigFragment.m_isConnected) && UsageInfoFragment.ojoDerEncender){
                        if(MQTTConfigFragment.conectado && transcurridoDesdeEnviado>= 2000){
                            cejasObject.put("state","ON")
                            publish(MQTTConfigFragment.publishTopic, cejasObject.toString())
                            lastPublishedMessage = "EnviadoON"
                            ultimoEnviado = System.currentTimeMillis()
                        }
                        if(BTConfigFragment.m_isConnected && transcurridoDesdeBT>= 5000){
                            sendCommand("A")
                            lastPublishedMessage = "Cambio Plato"
                            ultimoBT = System.currentTimeMillis()
                        }
                        //ultimoEncendido = System.currentTimeMillis()
                    }
                    else if((MQTTConfigFragment.conectado || BTConfigFragment.m_isConnected) && UsageInfoFragment.ojoDerApagar){
                        if(MQTTConfigFragment.conectado && transcurridoDesdeEnviado >= 2000){
                            cejasObject.put("state","OFF")
                            publish(MQTTConfigFragment.publishTopic, cejasObject.toString())
                            lastPublishedMessage = "EnviadoOFF"
                            ultimoEnviado = System.currentTimeMillis()
                        }
                        if(BTConfigFragment.m_isConnected && transcurridoDesdeBT >= 5000){
                            sendCommand("B")
                            lastPublishedMessage = "Dar de Comer"
                            ultimoBT = System.currentTimeMillis()
                        }
                        //ultimoApagado = System.currentTimeMillis()
                    }
                    else if(MQTTConfigFragment.conectado && transcurridoDesdeConexion >= 5000 && UsageInfoFragment.ojoDerConexion){
                        //val fragmentA = requireActivity().supportFragmentManager.findFragmentByTag("MQTTConfigFragment")
                        //val fragmentA = fragmentManager?.findFragmentByTag("MQTTConfigFragment()") as? MQTTConfigFragment
                        Log.d("Intento MQTT", "Entró al if de desconexión con gesto")
                       // if (fragmentA != null && fragmentA is MQTTConfigFragment) {
                        Log.d("Intento MQTT", "Debería estar desconectando")
                        MQTTConfigFragment().desconexionMQTT()
                        Toast.makeText(requireContext(),"Desconectado",Toast.LENGTH_SHORT).show()
                       // }
                        ultimaConexion = System.currentTimeMillis()
                    }
                    else if(BTConfigFragment.m_isConnected && transcurridoDesdeConexion >=5000 && UsageInfoFragment.ojoDerConexion){
                        BTConfigFragment().disconnect()
                        Toast.makeText(requireContext(),"Desconectado de Bluetooth",Toast.LENGTH_SHORT).show()
                        ultimaConexion = System.currentTimeMillis()
                    }
                    else if(!MQTTConfigFragment.conectado && transcurridoDesdeConexion >= 5000 && UsageInfoFragment.ojoDerConexion){
                        Log.d("Intento MQTT", "Entró al if de conexión con gesto")
                        //if (fragmentA != null && fragmentA is MQTTConfigFragment) {
                        Log.d("Intento MQTT", "Intentando conexión")
                        if (!MQTToBT) { //Si lo último conectado fue BT
                            MQTToBT = true //Porque se intentó acceder a MQTT, aunque no se haya metido direcciones se pone para que el siguiente sea Bluetooth
                            //Para ver si puedo conectarme a la primera
                            MQTTConfigFragment.direccion = ConexionBBDDSQLite(requireContext()).getDnsById(1).toString()
                            MQTTConfigFragment.puerto = ConexionBBDDSQLite(requireContext()).getPuertoById(1).toString()
                            MQTTConfigFragment.serverUri = "tcp://${MQTTConfigFragment.direccion}:${MQTTConfigFragment.puerto}"
                            MQTTConfigFragment.usuarioMQTT = ConexionBBDDSQLite(requireContext()).getUserNameById(1).toString()
                            MQTTConfigFragment.contrasenaMQTT = ConexionBBDDSQLite(requireContext()).getContrasenaById(1).toString()
                            Log.d("IntentoConexionALaPrimera", "URI:${MQTTConfigFragment.serverUri}")
                            Toast.makeText(requireContext(),"Intentando conexión MQTT",Toast.LENGTH_SHORT).show()

                            MQTTConfigFragment().conexionMQTT(requireContext(),MQTTConfigFragment.serverUri) { exitosa ->
                                if(exitosa){
                                    requireActivity().runOnUiThread {
                                        Toast.makeText(requireContext(), "Conexión exitosa", Toast.LENGTH_LONG).show()
                                    }
                                } else {
                                    val handler = Handler(Looper.getMainLooper())
                                    handler.post {
                                        Toast.makeText(requireContext(), "Conexión fallida, vuelve a intentarlo", Toast.LENGTH_LONG).show()
                                    }
                                }
                            }
                        } else {
                            MQTToBT = false
                            val BTpreferences = requireContext().getSharedPreferences("BTPreferences", Context.MODE_PRIVATE)
                            BTConfigFragment.m_address = BTpreferences.getString("direccionBTGuardada", "").toString()
                            Log.d("Bluetooth", "DireccionBT obtenida del preferences: ${BTConfigFragment.m_address}")
                            Toast.makeText(requireContext(),"Intentando conexión Bluetooth",Toast.LENGTH_SHORT).show()
                            if(BTConfigFragment.m_address!=""){
                                var mBtAdapter = (requireContext().getSystemService(Context.BLUETOOTH_SERVICE) as BluetoothManager).adapter
                                val device: BluetoothDevice = mBtAdapter.getRemoteDevice(BTConfigFragment.m_address)
                                if (ActivityCompat.checkSelfPermission(requireContext(), Manifest.permission.BLUETOOTH_CONNECT) != PackageManager.PERMISSION_GRANTED) {
                                    ActivityCompat.requestPermissions(
                                        requireActivity(), arrayOf(
                                            Manifest.permission.BLUETOOTH,
                                            Manifest.permission.BLUETOOTH_CONNECT,
                                            Manifest.permission.BLUETOOTH_ADMIN,
                                            Manifest.permission.BLUETOOTH_SCAN,
                                            Manifest.permission.BLUETOOTH_ADVERTISE,
                                            Manifest.permission.BLUETOOTH_PRIVILEGED,
                                            Manifest.permission.MANAGE_DEVICE_POLICY_BLUETOOTH,
                                        ), 1)
                                }
                                connectToDevice(device)
                            } else {
                                Toast.makeText(requireContext(),"No hay dirección de dispositivo BT registrada",Toast.LENGTH_SHORT).show()
                            }
                        }
                      //  }
                        ultimaConexion = System.currentTimeMillis()
                    }
                    else if(transcurridoDesdeCambio>=5000 && UsageInfoFragment.ojoDerCambiar){
                        if(topicoActual<MQTTConfigFragment.numeroTopics){
                            topicoActual=topicoActual+1
                        } else {
                            topicoActual=1
                        }
                        ultimoCambio = System.currentTimeMillis()
                    }
                }
                if((distanciaGesto/distanciaExt)>UsageInfoFragment.umbralOjoDer*0.01){
                //if((distanciaGesto/distanciaExt)>0.03){
                    seRealiza = 0
                    binding.progressBarOjoDer.progress = 0
                    ObjectAnimator.ofInt(binding.progressBarOjoDer,"progress", 500)
                        .setDuration(5000)
                        .start()
                    binding.progressBarOjoDer2.progress = 0
                    ObjectAnimator.ofInt(binding.progressBarOjoDer2,"progress", 500)
                        .setDuration(5000)
                        .start()
                    handlerOjoDer.removeCallbacksAndMessages(null)
                    binding.textView9.text = "Gesto ojo derecho:"
                    binding.textViewProgressBarOjoDer2.text = "Gesto ojo derecho:"
//                    if(MQTTConfigFragment.conectado&&lastPublishedMessageOjoDer!="EnviadoAbierto")
//                       // publish(MQTTConfigFragment.publishTopic4, "Abierto")
//                    lastPublishedMessageOjoDer="EnviadoAbierto"
//                    mActivity?.runOnUiThread {
//                        binding.textView4.text = "Ojo derecho abierto"
//                    }
                } else{
                    seRealiza = 1
                    if(seRealiza==1){
                        handlerOjoDer.postDelayed(runnableOjoDerechoCerrado, 5000)
                    } else {
                        handlerOjoDer.removeCallbacks(runnableOjoDerechoCerrado)
                    }
//                    mActivity?.runOnUiThread {
//                        binding.textView4.text = "Ojo derecho cerrado"
//                    }
                }
            }

            else->{
                val runnableOjoIzquierdoCerrado = Runnable {
                    binding.textView10.text = "Ojo izquierdo cerrado durante 5 segundos consecutivos"
                    binding.textViewProgressBarOjoIzq2.text = "Ojo izquierdo cerrado durante 5 segundos consecutivos"
                    //transcurridoDesdeEncendido = System.currentTimeMillis() - ultimoEncendido
                    //transcurridoDesdeApagado = System.currentTimeMillis() - ultimoApagado
                    transcurridoDesdeEnviado = System.currentTimeMillis() - ultimoEnviado
                    transcurridoDesdeBT = System.currentTimeMillis() - ultimoBT
                    transcurridoDesdeConexion = System.currentTimeMillis() - ultimaConexion
                    transcurridoDesdeCambio = System.currentTimeMillis() - ultimoCambio
                    if((MQTTConfigFragment.conectado || BTConfigFragment.m_isConnected) && UsageInfoFragment.ojoIzqEncender){
                        if(MQTTConfigFragment.conectado && transcurridoDesdeEnviado>= 2000){
                            cejasObject.put("state","ON")
                            publish(MQTTConfigFragment.publishTopic, cejasObject.toString())
                            lastPublishedMessage = "EnviadoON"
                            ultimoEnviado = System.currentTimeMillis()
                        }
                        if(BTConfigFragment.m_isConnected && transcurridoDesdeBT>= 5000){
                            sendCommand("A")
                            lastPublishedMessage = "Cambio Plato"
                            ultimoBT = System.currentTimeMillis()
                        }
                        //ultimoEncendido = System.currentTimeMillis()
                    }
                    else if((MQTTConfigFragment.conectado || BTConfigFragment.m_isConnected) && UsageInfoFragment.ojoIzqApagar){
                        if(MQTTConfigFragment.conectado && transcurridoDesdeEnviado>= 2000){
                            cejasObject.put("state","OFF")
                            publish(MQTTConfigFragment.publishTopic, cejasObject.toString())
                            lastPublishedMessage = "EnviadoOFF"
                            ultimoEnviado = System.currentTimeMillis()
                        }
                        if(BTConfigFragment.m_isConnected && transcurridoDesdeBT>= 5000){
                            sendCommand("B")
                            lastPublishedMessage = "Dar de Comer"
                            ultimoBT = System.currentTimeMillis()
                        }
                        //ultimoApagado = System.currentTimeMillis()
                    }
                    else if(MQTTConfigFragment.conectado && transcurridoDesdeConexion >= 5000 && UsageInfoFragment.ojoIzqConexion){
                        MQTTConfigFragment().desconexionMQTT()
                        Toast.makeText(requireContext(),"Desconectado",Toast.LENGTH_SHORT).show()
                        ultimaConexion = System.currentTimeMillis()
                    }
                    else if(BTConfigFragment.m_isConnected && transcurridoDesdeConexion >=5000 && UsageInfoFragment.ojoIzqConexion){
                        BTConfigFragment().disconnect()
                        Toast.makeText(requireContext(),"Desconectado de Bluetooth",Toast.LENGTH_SHORT).show()
                        ultimaConexion = System.currentTimeMillis()
                    }
                    else if(!MQTTConfigFragment.conectado && transcurridoDesdeConexion >= 5000 && UsageInfoFragment.ojoIzqConexion){

                        if (!MQTToBT) { //Si lo último conectado fue BT
                            MQTToBT = true //Porque se intentó acceder a MQTT, aunque no se haya metido direcciones se pone para que el siguiente sea Bluetooth
                            //Para ver si puedo conectarme a la primera
                            MQTTConfigFragment.direccion = ConexionBBDDSQLite(requireContext()).getDnsById(1).toString()
                            MQTTConfigFragment.puerto = ConexionBBDDSQLite(requireContext()).getPuertoById(1).toString()
                            MQTTConfigFragment.serverUri = "tcp://${MQTTConfigFragment.direccion}:${MQTTConfigFragment.puerto}"
                            MQTTConfigFragment.usuarioMQTT = ConexionBBDDSQLite(requireContext()).getUserNameById(1).toString()
                            MQTTConfigFragment.contrasenaMQTT = ConexionBBDDSQLite(requireContext()).getContrasenaById(1).toString()
                            Log.d("IntentoConexionALaPrimera", "URI:${MQTTConfigFragment.serverUri}")
                            Toast.makeText(requireContext(),"Intentando conexión MQTT",Toast.LENGTH_SHORT).show()

                            MQTTConfigFragment().conexionMQTT(requireContext(),MQTTConfigFragment.serverUri) { exitosa ->
                                if(exitosa){
                                    requireActivity().runOnUiThread {
                                        Toast.makeText(requireContext(), "Conexión exitosa", Toast.LENGTH_LONG).show()
                                    }
                                } else {
                                    val handler = Handler(Looper.getMainLooper())
                                    handler.post {
                                        Toast.makeText(requireContext(), "Conexión fallida, vuelve a intentarlo", Toast.LENGTH_LONG).show()
                                    }
                                }
                            }
                        } else {
                            MQTToBT = false
                            val BTpreferences = requireContext().getSharedPreferences("BTPreferences", Context.MODE_PRIVATE)
                            BTConfigFragment.m_address = BTpreferences.getString("direccionBTGuardada", "").toString()
                            Log.d("Bluetooth", "DireccionBT obtenida del preferences: ${BTConfigFragment.m_address}")
                            Toast.makeText(requireContext(),"Intentando conexión Bluetooth",Toast.LENGTH_SHORT).show()
                            if(BTConfigFragment.m_address!=""){
                                var mBtAdapter = (requireContext().getSystemService(Context.BLUETOOTH_SERVICE) as BluetoothManager).adapter
                                val device: BluetoothDevice = mBtAdapter.getRemoteDevice(BTConfigFragment.m_address)
                                if (ActivityCompat.checkSelfPermission(requireContext(), Manifest.permission.BLUETOOTH_CONNECT) != PackageManager.PERMISSION_GRANTED) {
                                    ActivityCompat.requestPermissions(
                                        requireActivity(), arrayOf(
                                            Manifest.permission.BLUETOOTH,
                                            Manifest.permission.BLUETOOTH_CONNECT,
                                            Manifest.permission.BLUETOOTH_ADMIN,
                                            Manifest.permission.BLUETOOTH_SCAN,
                                            Manifest.permission.BLUETOOTH_ADVERTISE,
                                            Manifest.permission.BLUETOOTH_PRIVILEGED,
                                            Manifest.permission.MANAGE_DEVICE_POLICY_BLUETOOTH,
                                        ), 1)
                                }
                                connectToDevice(device)
                            } else {
                                Toast.makeText(requireContext(),"No hay dirección de dispositivo BT registrada",Toast.LENGTH_SHORT).show()
                            }
                        }
                        ultimaConexion = System.currentTimeMillis()
                    }
                    else if(transcurridoDesdeCambio>=5000 && UsageInfoFragment.ojoIzqCambiar){
                        if(topicoActual<MQTTConfigFragment.numeroTopics){
                            topicoActual=topicoActual+1
                        } else {
                            topicoActual=1
                        }
                        ultimoCambio = System.currentTimeMillis()
                    }
                }
                if((distanciaGesto/distanciaExt)>UsageInfoFragment.umbralOjoIzq*0.01){
                    seRealiza = 0
                    binding.progressBarOjoIzq.progress = 0
                    ObjectAnimator.ofInt(binding.progressBarOjoIzq,"progress", 500)
                        .setDuration(5000)
                        .start()
                    binding.progressBarOjoIzq2.progress = 0
                    ObjectAnimator.ofInt(binding.progressBarOjoIzq2,"progress", 500)
                        .setDuration(5000)
                        .start()
                    handlerOjoIzq.removeCallbacksAndMessages(null)
                    binding.textView10.text = "Gesto ojo izquierdo:"
                    binding.textViewProgressBarOjoIzq2.text = "Gesto ojo izquierdo:"
//                    if(MQTTConfigFragment.conectado && lastPublishedMessageOjoIzq!="EnviadoAbierto")
//                      //  publish(MQTTConfigFragment.publishTopic5, "Abierto")
//                    lastPublishedMessageOjoIzq="EnviadoAbierto"
//                    mActivity?.runOnUiThread {
//                        binding.textView5.text = "Ojo izquierdo abierto"
//                    }
                } else{
                    seRealiza = 1
                    if(seRealiza==1){
                        handlerOjoIzq.postDelayed(runnableOjoIzquierdoCerrado, 5000)
                    } else {
                        handlerOjoIzq.removeCallbacks(runnableOjoIzquierdoCerrado)
                    }
//                    mActivity?.runOnUiThread {
//                        binding.textView5.text = "Ojo izquierdo cerrado"
//                    }
                }
            }
        }
        return seRealiza
    }







    //GPT para Bluetooth
    private fun connectToDevice(device: BluetoothDevice) {
        CoroutineScope(Dispatchers.IO).launch {
            try {
                if (ActivityCompat.checkSelfPermission(requireContext(), Manifest.permission.BLUETOOTH_CONNECT) != PackageManager.PERMISSION_GRANTED) {
                    ActivityCompat.requestPermissions(
                        requireActivity(), arrayOf(
                            Manifest.permission.BLUETOOTH,
                            Manifest.permission.BLUETOOTH_CONNECT,
                            Manifest.permission.BLUETOOTH_ADMIN,
                            Manifest.permission.BLUETOOTH_SCAN,
                            Manifest.permission.BLUETOOTH_ADVERTISE,
                            Manifest.permission.BLUETOOTH_PRIVILEGED,
                            Manifest.permission.MANAGE_DEVICE_POLICY_BLUETOOTH,
                        ),
                        1
                    )
                }
                BTConfigFragment.m_bluetoothSocket = device.createInsecureRfcommSocketToServiceRecord(BTConfigFragment.m_myUUID) // BTConfigFragment.m_bluetoothSocket = device.createRfcommSocketToServiceRecord(BTConfigFragment.m_myUUID)
                BTConfigFragment.m_bluetoothSocket!!.connect()
                BTConfigFragment.m_isConnected=true
                withContext(Dispatchers.Main) {
                    Toast.makeText(requireContext(), "Conexión Bluetooth exitosa", Toast.LENGTH_SHORT).show()
                }
                Log.d("Bluetooth", "UUID:${BTConfigFragment.m_myUUID}")
                Log.d("Bluetooth", "Direccion:${BTConfigFragment.m_address}")
            } catch (e: IOException) {
                Log.e("Bluetooth", "Error al conectar con el dispositivo Bluetooth", e)
                withContext(Dispatchers.Main) {
                    Toast.makeText(requireContext(), "Error al conectar con el dispositivo Bluetooth", Toast.LENGTH_SHORT).show()
                }
            } finally {
                if (BTConfigFragment.m_bluetoothSocket != null && !BTConfigFragment.m_bluetoothSocket!!.isConnected) {
                    try {
                        BTConfigFragment.m_bluetoothSocket!!.close()
                    } catch (e: IOException) {
                        Log.e("Bluetooth", "Error al cerrar el socket Bluetooth: ", e)
                    }
                }
            }
        }
    }













    //Función para cambiar la posición de las progressBars para que el usuario no tenga que mirar a la derecha
    fun actualizarPosicionProgressBars(posicionesX: MutableList<Float>, posicionesY: MutableList<Float>, factorX: Float, width: Int, height: Int, isTablet: Boolean) {

        // Actualiza las posiciones de las TextViews y las ProgressBars en función de las coordenadas de la cara detectada

        if(isTablet){
            binding.linearLayoutProgressBar.x = (-posicionesX[9] * factorX + width) - (binding.progressBarCejas.width / 4)
            binding.linearLayoutProgressBar.y = ((posicionesY[10] * factorX) - (1220 - height) * factorX)

            binding.linearLayoutProgressBar2.x = (-posicionesX[200] * factorX + width) - (binding.progressBarCejas.width / 4)
            binding.linearLayoutProgressBar2.y = ((posicionesY[152] * factorX) - (1188 - height) * factorX)

            binding.linearLayoutProgressBar3.x = (-posicionesX[200] * factorX + width) - (binding.progressBarCejas.width / 4)
            binding.linearLayoutProgressBar3.y = ((posicionesY[152] * factorX) - (1140 - height) * factorX)

            binding.linearLayoutProgressBar4.x = (-posicionesX[234] * factorX + width)
            binding.linearLayoutProgressBar4.y = ((posicionesY[230] * factorX) - (1188 - height) * factorX)

            binding.linearLayoutProgressBar5.x = (-posicionesX[454] * factorX + width) - (binding.progressBarCejas.width / 2)
            binding.linearLayoutProgressBar5.y = ((posicionesY[253] * factorX) - (1188 - height) * factorX)
        } else {
            binding.linearLayoutProgressBar.x = (-posicionesX[9] * factorX + width) - (binding.progressBarCejas.width / 4)
            binding.linearLayoutProgressBar.y = ((posicionesY[10] * factorX) + 300)

            binding.linearLayoutProgressBar2.x = (-posicionesX[200] * factorX + width) - (binding.progressBarCejas.width / 4)
            binding.linearLayoutProgressBar2.y = ((posicionesY[152] * factorX) + 800)

            binding.linearLayoutProgressBar3.x = (-posicionesX[200] * factorX + width) - (binding.progressBarCejas.width / 4)
            binding.linearLayoutProgressBar3.y = ((posicionesY[152] * factorX) + 950)

            binding.linearLayoutProgressBar4.x = (-posicionesX[234] * factorX + width) - 25
            binding.linearLayoutProgressBar4.y = ((posicionesY[230] * factorX) + 500)

            binding.linearLayoutProgressBar5.x = (-posicionesX[454] * factorX + width) - 650
            binding.linearLayoutProgressBar5.y = ((posicionesY[253] * factorX) + 500)
        }
    }


    fun selectorColor(cejas:Int, boca:Int): Paint{

        val paint = Paint().apply {
            color = Color.argb(0xff, 0xff, 0x0, 0x00) // Color de los puntos
            style = Paint.Style.FILL
            strokeWidth = 5f // Grosor del punto
        }

        if(UsageInfoFragment.visualizarDaltonismo){

            binding.linearLayoutPrincipal.setBackgroundColor(Color.MAGENTA)
            binding.textViewConect.setTextColor(Color.BLACK)
            binding.textViewGestos.setTextColor(Color.BLACK)
            binding.textViewUltimoMensaje.setTextColor(Color.BLACK)
            binding.textViewTopicoActual.setTextColor(Color.BLACK)

            binding.linearLayoutUmbrales?.setBackgroundColor(Color.rgb(0, 128, 128))
            binding.textViewUmbralCejas?.setTextColor(Color.rgb(117, 255, 255))
            binding.textViewUmbralBoca?.setTextColor(Color.rgb(117, 255, 255))
            binding.textViewUmbralOjoDer?.setTextColor(Color.rgb(117, 255, 255))
            binding.textViewUmbralOjoIzq?.setTextColor(Color.rgb(117, 255, 255))

            if(cejas==1 && boca==1)
                paint.color = Color.argb(0xff, 0x64, 0x8F, 0xff)
            else if(cejas==0 && boca==1)
                paint.color = Color.argb(0xff, 0xDC, 0x26, 0x7f)
            else if(cejas==1 && boca==0)
                paint.color = Color.argb(0xff,0xfe,0x61,0x0)
            else
                paint.color = Color.argb(0xff,0xff,0xb0,0x0)
        } else {

            binding.linearLayoutPrincipal.setBackgroundColor(Color.YELLOW)
            binding.textViewConect.setTextColor(Color.RED)
            binding.textViewGestos.setTextColor(Color.RED)
            binding.textViewUltimoMensaje.setTextColor(Color.RED)
            binding.textViewTopicoActual.setTextColor(Color.RED)

            binding.linearLayoutUmbrales?.setBackgroundColor(Color.rgb(53,188,59))
            binding.textViewUmbralCejas?.setTextColor(Color.BLACK)
            binding.textViewUmbralBoca?.setTextColor(Color.BLACK)
            binding.textViewUmbralOjoDer?.setTextColor(Color.BLACK)
            binding.textViewUmbralOjoIzq?.setTextColor(Color.BLACK)

            if(cejas==1 && boca==1)
                paint.color = Color.argb(0xff, 0xff, 0x0, 0x0)
            else if(cejas==0 && boca==1)
                paint.color = Color.argb(0xff, 0x0, 0xff, 0x0)
            else if(cejas==1 && boca==0)
                paint.color = Color.argb(0xff,0x0,0x0,0xff)
            else
                paint.color = Color.argb(0xff,0xff,0xff,0x0)
        }
        return paint
    }


    fun selectorColorCejas(cejas:Int): Paint{

        val paintCejas = Paint().apply {
            color = Color.argb(0xff, 0xff, 0xff, 0xff) // Color de los puntos
            style = Paint.Style.FILL
            strokeWidth = 5f // Grosor del punto
        }
        if(cejas==1) //Si está bajado lo pinto de negro, si está subido, de blanco
            paintCejas.color = Color.argb(0xff, 0x0, 0x0, 0x0)
        else
            paintCejas.color = Color.argb(0xff, 0xf0, 0xf0, 0xf0)

        return paintCejas
    }


    fun selectorColorBoca(boca: Int, posBoca: Int):Paint{
        //Color de la boca en función del gesto
        val paintBoca = Paint().apply {
            color = Color.argb(0xff, 0xff, 0xff, 0xff) // Color de los puntos
            style = Paint.Style.FILL
            strokeWidth = 5f // Grosor del punto
        }
//        when (posBoca) {
//            0 -> paintBoca.color = Color.argb(0xff, 0x0, 0x0, 0x0)
//            1 -> paintBoca.color = Color.argb(0xff, 0x28, 0x28, 0x28)
//            2 -> paintBoca.color = Color.argb(0xff,0x50,0x50,0x50)
//            3 -> paintBoca.color = Color.argb(0xff,0x78,0x78,0x78)
//            4 -> paintBoca.color = Color.argb(0xff, 0xa0, 0xa0, 0xa0)
//            5 -> paintBoca.color = Color.argb(0xff,0xc8,0xc8,0xc8)
//            6 -> paintBoca.color = Color.argb(0xff,0xf0,0xf0,0xf0)
//        }

        if(boca==1) //Si está cerrada la pinto de negro, si está abierta, de blanco
            paintBoca.color = Color.argb(0xff, 0x0, 0x0, 0x0)
        else
            paintBoca.color = Color.argb(0xff, 0xf0, 0xf0, 0xf0)


        return paintBoca
    }


    fun selectorColorOjoDer(ojoDer:Int): Paint{

        val paintOjoDer = Paint().apply {
            color = Color.argb(0xff, 0xff, 0xff, 0xff) // Color de los puntos
            style = Paint.Style.FILL
            strokeWidth = 5f // Grosor del punto
        }
        if(ojoDer==0) //Si está abierto lo pinto de negro, si está cerrado, de blanco
            paintOjoDer.color = Color.argb(0xff, 0x0, 0x0, 0x0)
        else
            paintOjoDer.color = Color.argb(0xff, 0xf0, 0xf0, 0xf0)

        return paintOjoDer
    }


    fun selectorColorOjoIzq(ojoIzq:Int): Paint{

        val paintOjoIzq = Paint().apply {
            color = Color.argb(0xff, 0xff, 0xff, 0xff) // Color de los puntos
            style = Paint.Style.FILL
            strokeWidth = 5f // Grosor del punto
        }
        if(ojoIzq==0)
            paintOjoIzq.color = Color.argb(0xff, 0x0, 0x0, 0x0)
        else
            paintOjoIzq.color = Color.argb(0xff, 0xf0, 0xf0, 0xf0)

        return paintOjoIzq
    }


    fun dibujaPuntos(faceMeshpoints: MutableList<FaceMeshPoint>, overlayView: View, bitmap: Bitmap, canvas: Canvas, paint: Paint, isTablet: Boolean, factorX: Float, factorY: Float, width: Int, height: Int){

        mActivity?.runOnUiThread {
            for (faceMeshpoint in faceMeshpoints) {
                val position = faceMeshpoint.position
                if(isTablet) { //Si es tablet
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) { //Si está en horizontal
                        canvas.drawCircle((-position.x * factorX) + width, (position.y * factorX) - (1188 - height) * factorX, 5f, paint)
                    } else { //Si está en vertical
                        canvas.drawCircle(((-position.x * 2.8) + width + 40 * factorX).toFloat(), ((position.y * 2.8).toFloat()) + 10 * factorX, 5f, paint)
                    }
                } else { //Si es móvil
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
                        canvas.drawCircle((-position.x * factorX) + width, (position.y * factorX) - (1177 - height) * factorX, 5f, paint)
                    } else { //Si está en vertical
                        canvas.drawCircle((-position.x * 3.3f) + width + (80 * 3.2f), (position.y * 3.3f) + 30, 5f, paint)
                    }
                }
            }
            overlayView.background = BitmapDrawable(resources,bitmap)
        }
    }

    //Función creada para que no hubiera un if(Daltonismo) en el dibuja puntos y así aprovechar
    //y que no fuera algo específico de la opción de daltonismo
    fun dibujaTextos(faceMeshpoints: MutableList<FaceMeshPoint>, overlayView: View, bitmap: Bitmap, canvas: Canvas, paint: Paint, isTablet: Boolean, factorX: Float, factorY: Float, width: Int, height: Int, textoAMostrar: String, tamano: Double){

        val paintTexto = Paint().apply {
            color = paint.color
            style = Paint.Style.FILL
            strokeWidth = 5f // Grosor del punto
        }
        paintTexto.textSize = 30f
        if(UsageInfoFragment.visualizarPuntos) {
            canvas.drawText("$textoAMostrar", (-faceMeshpoints[251].position.x * factorX + width) + (390 * (tamano * 0.6)).toFloat(), (faceMeshpoints[251].position.y * factorX) - (1188 - height) * factorX, paintTexto)
        }
        else{
            paintTexto.color = Color.BLACK
            canvas.drawText("$textoAMostrar", (-faceMeshpoints[251].position.x * factorX + width) + (390 * (tamano*0.6)).toFloat(), (faceMeshpoints[251].position.y * factorX) - (1188 - height) * factorX, paintTexto)
        }
    }


    fun dibujaPuntosCejas(faceMeshpoints: MutableList<FaceMeshPoint>, canvas: Canvas, paintCejas: Paint, isTablet: Boolean, factorX: Float, factorY: Float, width: Int, height: Int){
        //val puntosCejasDer = setOf(276, 283, 283, 282, 282, 295, 295, 285, 300, 293, 293, 334, 334, 296, 296, 336)
        //val puntosCejasIzq = setOf(46, 53, 53, 52, 52, 65, 65, 55, 70, 63, 63, 105, 105, 66, 66, 107)

        val puntosCejas = setOf(276, 283, 283, 282, 282, 295, 295, 285, 300, 293, 293, 334, 334, 296, 296, 336, 46, 53, 53, 52, 52, 65, 65, 55, 70, 63, 63, 105, 105, 66, 66, 107)

        for (i in puntosCejas){
            val position = faceMeshpoints[i].position
            if(UsageInfoFragment.visualizarDaltonismo){
                if(isTablet) { //Si es tablet
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) { //Si está en horizontal
                        val path = Path()
                        path.moveTo((-position.x * factorX) + width, (position.y * factorX) - (1188 - height) * factorX + 5)
                        path.lineTo((-position.x * factorX) + width - 5, (position.y * factorX) - (1188 - height) * factorX - 5)
                        path.lineTo((-position.x * factorX) + width + 5, (position.y * factorX) - (1188 - height) * factorX - 5)
                        path.close()
                        canvas.drawPath(path, paintCejas)
                    } else { //Si está en vertical
                        val path = Path()
                        path.moveTo(((-position.x * 2.8) + width + 40 * factorX).toFloat(), ((position.y * 2.8).toFloat()) + 10 * factorX)
                        path.lineTo(((-position.x * 2.8) + width + 40 * factorX).toFloat() - 5, ((position.y * 2.8).toFloat()) + 10 * factorX + 5)
                        path.lineTo(((-position.x * 2.8) + width + 40 * factorX).toFloat() + 5, ((position.y * 2.8).toFloat()) + 10 * factorX + 5)
                        path.close()
                        canvas.drawPath(path, paintCejas)
                    }
                } else { //Si es móvil
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
                        val path = Path()
                        path.moveTo((-position.x * factorX) + width, (position.y * factorX) - (1177 - height) * factorX + 5)
                        path.lineTo((-position.x * factorX) + width - 5, (position.y * factorX) - (1177 - height) * factorX - 5)
                        path.lineTo((-position.x * factorX) + width + 5, (position.y * factorX) - (1177 - height) * factorX - 5)
                        path.close()
                        canvas.drawPath(path, paintCejas)
                    } else { //Si está en vertical
                        val path = Path()
                        path.moveTo((-position.x * 3.3f) + width + (80 * 3.2f), (position.y * 3.3f) + 30)
                        path.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) - 5, (position.y * 3.3f) + 30 + 5)
                        path.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + 5, (position.y * 3.3f) + 30 + 5)
                        path.close()
                        canvas.drawPath(path, paintCejas)
                    }
                }
            } else {
                if(isTablet) { //Si es tablet
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) { //Si está en horizontal
                        canvas.drawCircle((-position.x * factorX) + width, (position.y * factorX) - (1188 - height) * factorX, 5f, paintCejas)
                    } else { //Si está en vertical
                        canvas.drawCircle(((-position.x * 2.8) + width + 40 * factorX).toFloat(), ((position.y * 2.8).toFloat()) + 10 * factorX, 5f, paintCejas)
                    }
                } else { //Si es móvil
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
                        canvas.drawCircle((-position.x * factorX) + width, (position.y * factorX) - (1177 - height) * factorX, 5f, paintCejas)
                    } else { //Si está en vertical
                        canvas.drawCircle((-position.x * 3.3f) + width + (80 * 3.2f), (position.y * 3.3f) + 30 , 5f, paintCejas)
                        //canvas.drawCircle((-position.x * factorY) + width + (80 * factorY), (position.y * factorY), 5f, paintBoca)
                    }
                }
            }
        }
    }

    fun dibujaFlechaCejas(faceMeshpoints: MutableList<FaceMeshPoint>, canvas: Canvas, paintCejas: Paint, isTablet: Boolean, factorX: Float, factorY: Float, width: Int, height: Int, tamano: Double){

        val position = faceMeshpoints[10].position

        val pathTabletHorizontal = Path()
        // Definir la forma de la flecha
        pathTabletHorizontal.moveTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1188 - height) * factorX - (25f*(tamano*0.6)).toFloat()) // Punto inicial
        pathTabletHorizontal.lineTo((-position.x * factorX) + width - (50f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + (25f*(tamano*0.6)).toFloat()) // Línea izquierda
        pathTabletHorizontal.lineTo((-position.x * factorX) + width - (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + (25f * (tamano*0.6)).toFloat()) // Línea inferior corta
        pathTabletHorizontal.lineTo((-position.x * factorX) + width - (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + (75f * (tamano*0.6)).toFloat()) // Línea vertical
        pathTabletHorizontal.lineTo((-position.x * factorX) + width + (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + (75f * (tamano*0.6)).toFloat()) // Línea inferior larga
        pathTabletHorizontal.lineTo((-position.x * factorX) + width + (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + (25f * (tamano*0.6)).toFloat()) // Línea derecha
        pathTabletHorizontal.lineTo((-position.x * factorX) + width + (50f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + (25f * (tamano*0.6)).toFloat()) // Línea superior corta
        pathTabletHorizontal.lineTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1188 - height) * factorX - (25f * (tamano*0.6)).toFloat()) // Volver al punto inicial


        val pathTabletVertical = Path()
        // Definir la forma de la flecha
        pathTabletVertical.moveTo((-position.x * 2.8f) + width + 40 * factorX + 0f, (position.y * 2.8f) + 10 * factorX - (25f*(tamano*0.6)).toFloat()) // Punto inicial
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX - (50f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (25f*(tamano*0.6)).toFloat()) // Línea izquierda
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX - (25f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (25f * (tamano*0.6)).toFloat()) // Línea inferior corta
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX - (25f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (75f * (tamano*0.6)).toFloat()) // Línea vertical
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX + (25f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (75f * (tamano*0.6)).toFloat()) // Línea inferior larga
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX + (25f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (25f * (tamano*0.6)).toFloat()) // Línea derecha
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX + (50f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (25f * (tamano*0.6)).toFloat()) // Línea superior corta
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX + 0f, (position.y * 2.8f) + 10 * factorX - (25f * (tamano*0.6)).toFloat()) // Volver al punto inicial



//        val pathMovilHorizontal = Path()
//        // Definir la forma de la flecha
//        pathMovilHorizontal.moveTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1177 - height) * factorX - (25f*(tamano*0.6)).toFloat()) // Punto inicial
//        pathMovilHorizontal.lineTo((-position.x * factorX) + width - (50f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1177 - height) * factorX + (25f*(tamano*0.6)).toFloat()) // Línea izquierda
//        pathMovilHorizontal.lineTo((-position.x * factorX) + width - (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1177 - height) * factorX + (25f * (tamano*0.6)).toFloat()) // Línea inferior corta
//        pathMovilHorizontal.lineTo((-position.x * factorX) + width - (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1177 - height) * factorX + (75f * (tamano*0.6)).toFloat()) // Línea vertical
//        pathMovilHorizontal.lineTo((-position.x * factorX) + width + (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1177 - height) * factorX + (75f * (tamano*0.6)).toFloat()) // Línea inferior larga
//        pathMovilHorizontal.lineTo((-position.x * factorX) + width + (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1177 - height) * factorX + (25f * (tamano*0.6)).toFloat()) // Línea derecha
//        pathMovilHorizontal.lineTo((-position.x * factorX) + width + (50f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1177 - height) * factorX + (25f * (tamano*0.6)).toFloat()) // Línea superior corta
//        pathMovilHorizontal.lineTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1177 - height) * factorX - (25f * (tamano*0.6)).toFloat()) // Volver al punto inicial


        val pathMovilVertical = Path()
        // Definir la forma de la flecha
        pathMovilVertical.moveTo((-position.x * 3.3f) + width + (80 * 3.2f) + 0f, (position.y * 3.3f) + 30 - (25f*(tamano*0.6)).toFloat()) // Punto inicial
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) - (50f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (25f*(tamano*0.6)).toFloat()) // Línea izquierda
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) - (25f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (25f * (tamano*0.6)).toFloat()) // Línea inferior corta
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) - (25f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (75f * (tamano*0.6)).toFloat()) // Línea vertical
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + (25f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (75f * (tamano*0.6)).toFloat()) // Línea inferior larga
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + (25f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (25f * (tamano*0.6)).toFloat()) // Línea derecha
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + (50f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (25f * (tamano*0.6)).toFloat()) // Línea superior corta
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + 0f, (position.y * 3.3f) + 30 - (25f * (tamano*0.6)).toFloat()) // Volver al punto inicial





        if(isTablet) { //Si es tablet
            if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) { //Si está en horizontal
                canvas.drawPath(pathTabletHorizontal, paintCejas)
            } else { //Si está en vertical
                canvas.drawPath(pathTabletVertical, paintCejas)
            }
        } else { //Si es móvil
            if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
                canvas.drawPath(pathTabletHorizontal, paintCejas)
            } else { //Si está en vertical
                canvas.drawPath(pathMovilVertical, paintCejas)
            //canvas.drawCircle((-position.x * 3.3f) + width + (80 * 3.2f), (position.y * 3.3f) + 30, 5f, paint)
            }
        }
    }

    fun dibujaFlechaCejasPrueba(faceMeshpoints: MutableList<FaceMeshPoint>, canvas: Canvas, cejas: Int, isTablet: Boolean, factorX: Float, factorY: Float, width: Int, height: Int, tamano: Double){

        val paintNegro = Paint().apply {
            color = Color.argb(0xff, 0xff, 0xff, 0xff) // Color de los puntos
            style = Paint.Style.FILL
            strokeWidth = 5f // Grosor del punto
            color = Color.argb(0xff, 0x0, 0x0, 0x0)
        }
        val paintBlanco = Paint().apply {
            color = Color.argb(0xff, 0xff, 0xff, 0xff) // Color de los puntos
            style = Paint.Style.FILL
            strokeWidth = 5f // Grosor del punto
            color = Color.argb(0xff, 0xff, 0xff, 0xff)
        }

        val position = faceMeshpoints[10].position

        val pathSeg1 = Path()
        // Definir la forma de la flecha
        pathSeg1.moveTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1188 - height) * factorX - (25f*(tamano*0.6)).toFloat()) // Punto inicial
        pathSeg1.lineTo((-position.x * factorX) + width - (50f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + (25f*(tamano*0.6)).toFloat()) // Línea izquierda
        //pathSeg1.lineTo((-position.x * factorX) + width + 75f, (position.y * factorX) - (1188 - height) * factorX + 25f) // Línea inferior corta
        //pathSeg1.lineTo((-position.x * factorX) + width + 125f, (position.y * factorX) - (1188 - height) * factorX + 25f) // Línea derecha
        pathSeg1.lineTo((-position.x * factorX) + width + (50f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + (25f*(tamano*0.6)).toFloat()) // Línea superior corta
        pathSeg1.lineTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1188 - height) * factorX - (25f * (tamano*0.6)).toFloat()) // Volver al punto inicial

        val pathSeg2 = Path()
        // Definir la forma de la flecha
        pathSeg2.lineTo((-position.x * factorX) + width - (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + (25f * (tamano*0.6)).toFloat()) // Línea inferior corta
        pathSeg2.lineTo((-position.x * factorX) + width - (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + (75f * (tamano*0.6)).toFloat()) // Línea vertical
        pathSeg2.lineTo((-position.x * factorX) + width + (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + (75f * (tamano*0.6)).toFloat()) // Línea inferior larga
        pathSeg2.lineTo((-position.x * factorX) + width + (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + (25f * (tamano*0.6)).toFloat()) // Línea derecha
        pathSeg2.lineTo((-position.x * factorX) + width - (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + (25f * (tamano*0.6)).toFloat()) // Volver al punto inicial

        val handlerFlechaCejas1 = Handler(Looper.getMainLooper())
        val handlerFlechaCejas2 = Handler(Looper.getMainLooper())
        //Log.d("FlechaPrueba","HANDLER CREADO")

        /*
        CREO QUE SÍ SE DIBUJA EN BLANCO PERO SOLO 1 FRAME QUIZÁS
         */
//        val runnableFlechaCejas1 = Runnable {
//            mActivity?.runOnUiThread {
//                canvas.drawPath(pathSeg2, paintBlanco)   ///ESTA LÍNEA NO SE EJECUTA PERO EL LOG SÍ
//                //NO ENTIENDO NADA
//            }
//            Log.d("FlechaPrueba","DIBUJADO SEG2")
//        }
//        val runnableFlechaCejas2 = Runnable {
//            canvas.drawPath(pathSeg1, paintBlanco) //AQUÍ LO MISMO, NO SE EJECUTA PERO EL LOG SÍ APARECE
//            Log.d("FlechaPrueba","DIBUJADO SEG1")
//        }
    /*
    La intención es dibujar progresivamente la flecha de blanco a medida que se mantienen las cejas levantadas
    Intentando con handler.postDelayed no va y con timers tampoco
    */

        //canvas.drawPath(pathSeg2, paintNegro)
        //canvas.drawPath(pathSeg1, paintNegro)

        val runnableFlechaCejas1 = Runnable {
            dibujarSeg2=true
            Log.d("FlechaPrueba","PRIMER HANDLER")
            Log.d("FlechaPrueba", dibujarSeg2.toString())
        }
        val runnableFlechaCejas2 = Runnable {
            dibujarSeg1 = true
        }

        if(cejas==0){
            handlerFlechaCejas1.postDelayed(runnableFlechaCejas1,2500)
            handlerFlechaCejas2.postDelayed(runnableFlechaCejas2,5000)
        } else {
            dibujarSeg2=false
            dibujarSeg1=false
            handlerFlechaCejas1.removeCallbacks(runnableFlechaCejas1)
            handlerFlechaCejas2.removeCallbacks(runnableFlechaCejas2)
        }


        Log.d("FlechaPrueba", dibujarSeg2.toString())


        if(dibujarSeg2) {
            canvas.drawPath(pathSeg2, paintBlanco)
            Log.d("FlechaPrueba","DIBUJADO SEG2")
        } else {
            canvas.drawPath(pathSeg2, paintNegro)
            Log.d("FlechaPrueba","DIBUJADO SEG2 NEGRO")
        }
        if(dibujarSeg1){
            canvas.drawPath(pathSeg1, paintBlanco) //AQUÍ LO MISMO, NO SE EJECUTA PERO EL LOG SÍ APARECE
            Log.d("FlechaPrueba","DIBUJADO SEG1")
        } else {
            canvas.drawPath(pathSeg1, paintNegro)
            Log.d("FlechaPrueba","DIBUJADO SEG1 NEGRO")
        }


//Timers

//        if (cejas == 0) {
//            val timer = Timer()
//            timer.schedule(object : TimerTask() {
//                override fun run() {
//                    // Asegura que el dibujo se realice en el hilo de la interfaz de usuario
//                    Handler(Looper.getMainLooper()).post(runnableFlechaCejas1)
//                }
//            }, 2500)
//
//            val timer2 = Timer()
//            timer2.schedule(object : TimerTask() {
//                override fun run() {
//                    // Asegura que el dibujo se realice en el hilo de la interfaz de usuario
//                    Handler(Looper.getMainLooper()).post{canvas.drawPath(pathSeg1, paintBlanco)}
//                }
//            }, 5000)
//        } else {
//            canvas.drawPath(pathSeg2, paintNegro)
//            canvas.drawPath(pathSeg1, paintNegro)
//        }


        //pathSeg2.reset()
        //pathSeg1.reset()
    }


    fun dibujaPuntosBoca(faceMeshpoints: MutableList<FaceMeshPoint>, canvas: Canvas, paintBoca: Paint, isTablet: Boolean, factorX: Float, factorY: Float, width: Int, height: Int){

        val puntosBoca = setOf(61, 146, 146, 91, 91, 181, 181, 84, 84, 17, 17, 314, 314, 405, 405, 321,
            321, 375, 375, 291, 61, 185, 185, 40, 40, 39, 39, 37, 37, 0, 0, 267, 267,
            269, 269, 270, 270, 409, 409, 291, 78, 95, 95, 88, 88, 178, 178, 87, 87, 14,
            14, 317, 317, 402, 402, 318, 318, 324, 324, 308, 78, 191, 191, 80, 80, 81,
            81, 82, 82, 13, 13, 312, 312, 311, 311, 310, 310, 415, 415, 308)

        for (i in puntosBoca){
            val position = faceMeshpoints[i].position
            if(UsageInfoFragment.visualizarDaltonismo){
                if(isTablet) { //Si es tablet
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) { //Si está en horizontal
                        val path = Path()
                        path.moveTo((-position.x * factorX) + width, (position.y * factorX) - (1188 - height) * factorX + 5)
                        path.lineTo((-position.x * factorX) + width - 5, (position.y * factorX) - (1188 - height) * factorX - 5)
                        path.lineTo((-position.x * factorX) + width + 5, (position.y * factorX) - (1188 - height) * factorX - 5)
                        path.close()
                        canvas.drawPath(path, paintBoca)
                    } else { //Si está en vertical
                        val path = Path()
                        path.moveTo(((-position.x * 2.8) + width + 40 * factorX).toFloat(), ((position.y * 2.8).toFloat()) + 10 * factorX)
                        path.lineTo(((-position.x * 2.8) + width + 40 * factorX).toFloat() - 5, ((position.y * 2.8).toFloat()) + 10 * factorX + 5)
                        path.lineTo(((-position.x * 2.8) + width + 40 * factorX).toFloat() + 5, ((position.y * 2.8).toFloat()) + 10 * factorX + 5)
                        path.close()
                        canvas.drawPath(path, paintBoca)
                    }
                } else { //Si es móvil
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
                        val path = Path()
                        path.moveTo((-position.x * factorX) + width, (position.y * factorX) - (1177 - height) * factorX + 5)
                        path.lineTo((-position.x * factorX) + width - 5, (position.y * factorX) - (1177 - height) * factorX - 5)
                        path.lineTo((-position.x * factorX) + width + 5, (position.y * factorX) - (1177 - height) * factorX - 5)
                        path.close()
                        canvas.drawPath(path, paintBoca)
                    } else { //Si está en vertical
                        val path = Path()
                        path.moveTo((-position.x * 3.3f) + width + (80 * 3.2f), (position.y * 3.3f) + 30)
                        path.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) - 5, (position.y * 3.3f) + 30 + 5)
                        path.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + 5, (position.y * 3.3f) + 30 + 5)
                        path.close()
                        canvas.drawPath(path, paintBoca)
                    }
                }
            } else {
                if(isTablet) { //Si es tablet
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) { //Si está en horizontal
                        canvas.drawCircle((-position.x * factorX) + width, (position.y * factorX) - (1188 - height) * factorX, 5f, paintBoca)
                    } else { //Si está en vertical
                        canvas.drawCircle(((-position.x * 2.8) + width + 40 * factorX).toFloat(), ((position.y * 2.8).toFloat()) + 10 * factorX, 5f, paintBoca)
                    }
                } else { //Si es móvil
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
                        canvas.drawCircle((-position.x * factorX) + width, (position.y * factorX) - (1177 - height) * factorX, 5f, paintBoca)
                    } else { //Si está en vertical
                        canvas.drawCircle((-position.x * 3.3f) + width + (80 * 3.2f), (position.y * 3.3f) + 30 , 5f, paintBoca)
                        //canvas.drawCircle((-position.x * factorY) + width + (80 * factorY), (position.y * factorY), 5f, paintBoca)
                    }
                }
            }
        }
    }

    fun dibujaFlechaBoca(faceMeshpoints: MutableList<FaceMeshPoint>, canvas: Canvas, paintBoca: Paint, isTablet: Boolean, factorX: Float, factorY: Float, width: Int, height: Int, tamano: Double){
        val position = faceMeshpoints[175].position
        val path = Path()
        // Definir la forma de la flecha hacia abajo
        path.moveTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1188 - height) * factorX + (50f*(tamano*0.6).toFloat())) // Punto inicial
        path.lineTo((-position.x * factorX) + width - (50f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea izquierda
        path.lineTo((-position.x * factorX) + width - (25f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea inferior corta
        path.lineTo((-position.x * factorX) + width - (25f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX - (50f*(tamano*0.6).toFloat())) // Línea vertical
        path.lineTo((-position.x * factorX) + width + (25f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX - (50f*(tamano*0.6).toFloat())) // Línea inferior larga
        path.lineTo((-position.x * factorX) + width + (25f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea derecha
        path.lineTo((-position.x * factorX) + width + (50f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea superior corta
        path.lineTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1188 - height) * factorX + (50f*(tamano*0.6).toFloat())) // Volver al punto inicial


        val pathTabletVertical = Path()
        // Definir la forma de la flecha
        pathTabletVertical.moveTo((-position.x * 2.8f) + width + 40 * factorX + 0f, (position.y * 2.8f) + 10 * factorX + (50f*(tamano*0.6)).toFloat()) // Punto inicial
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX - (50f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (0f*(tamano*0.6)).toFloat()) // Línea izquierda
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX - (25f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (0f * (tamano*0.6)).toFloat()) // Línea inferior corta
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX - (25f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX - (50f * (tamano*0.6)).toFloat()) // Línea vertical
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX + (25f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX - (50f * (tamano*0.6)).toFloat()) // Línea inferior larga
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX + (25f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (0f * (tamano*0.6)).toFloat()) // Línea derecha
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX + (50f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (0f * (tamano*0.6)).toFloat()) // Línea superior corta
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX + 0f, (position.y * 2.8f) + 10 * factorX + (50f * (tamano*0.6)).toFloat()) // Volver al punto inicial


        val pathMovilVertical = Path()
        // Definir la forma de la flecha
        pathMovilVertical.moveTo((-position.x * 3.3f) + width + (80 * 3.2f) + 0f, (position.y * 3.3f) + 30 + (50f*(tamano*0.6)).toFloat()) // Punto inicial
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) - (50f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (0f*(tamano*0.6)).toFloat()) // Línea izquierda
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) - (25f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (0f * (tamano*0.6)).toFloat()) // Línea inferior corta
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) - (25f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 - (50f * (tamano*0.6)).toFloat()) // Línea vertical
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + (25f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 - (50f * (tamano*0.6)).toFloat()) // Línea inferior larga
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + (25f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (0f * (tamano*0.6)).toFloat()) // Línea derecha
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + (50f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (0f * (tamano*0.6)).toFloat()) // Línea superior corta
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + 0f, (position.y * 3.3f) + 30 + (50f * (tamano*0.6)).toFloat()) // Volver al punto inicial



        //canvas.drawPath(path, paintBoca)
        if(isTablet) { //Si es tablet
            if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) { //Si está en horizontal
                canvas.drawPath(path, paintBoca)
            } else { //Si está en vertical
                canvas.drawPath(pathTabletVertical, paintBoca)
            }
        } else { //Si es móvil
            if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
                canvas.drawPath(path, paintBoca)
            } else { //Si está en vertical
                canvas.drawPath(pathMovilVertical, paintBoca)
                //canvas.drawCircle((-position.x * 3.3f) + width + (80 * 3.2f), (position.y * 3.3f) + 30, 5f, paint)
            }
        }
    }


    fun dibujaFlechaBocaPrueba(faceMeshpoints: MutableList<FaceMeshPoint>, canvas: Canvas, boca: Int, isTablet: Boolean, factorX: Float, factorY: Float, width: Int, height: Int, tamano: Double){

        val paintNegro = Paint().apply {
            color = Color.argb(0xff, 0xff, 0xff, 0xff) // Color de los puntos
            style = Paint.Style.FILL
            strokeWidth = 5f // Grosor del punto
            color = Color.argb(0xff, 0x0, 0x0, 0x0)
        }
        val paintBlanco = Paint().apply {
            color = Color.argb(0xff, 0xff, 0xff, 0xff) // Color de los puntos
            style = Paint.Style.FILL
            strokeWidth = 5f // Grosor del punto
            color = Color.argb(0xff, 0xff, 0xff, 0xff)
        }


        val handlerFlechaCejas3 = Handler(Looper.getMainLooper())
        val handlerFlechaCejas4 = Handler(Looper.getMainLooper())

        val position = faceMeshpoints[175].position

        val pathSeg1 = Path()
        // Definir la forma de la flecha
        pathSeg1.moveTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1188 - height) * factorX + (50f*(tamano*0.6).toFloat())) // Punto inicial
        pathSeg1.lineTo((-position.x * factorX) + width - (50f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea izquierda
        pathSeg1.lineTo((-position.x * factorX) + width + (50f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea derecha
        pathSeg1.lineTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1188 - height) * factorX + (50f*(tamano*0.6).toFloat())) // Volver al punto inicial

        val pathSeg2 = Path()
        // Definir la forma de la flecha
        pathSeg2.lineTo((-position.x * factorX) + width - (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX - (50f * (tamano*0.6)).toFloat()) // Línea inferior corta
        pathSeg2.lineTo((-position.x * factorX) + width - (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + (0f * (tamano*0.6)).toFloat()) // Línea vertical
        pathSeg2.lineTo((-position.x * factorX) + width + (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX + (0f * (tamano*0.6)).toFloat()) // Línea inferior larga
        pathSeg2.lineTo((-position.x * factorX) + width + (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX - (50f * (tamano*0.6)).toFloat()) // Línea derecha
        pathSeg2.lineTo((-position.x * factorX) + width - (25f * (tamano*0.6)).toFloat(), (position.y * factorX) - (1188 - height) * factorX - (50f * (tamano*0.6)).toFloat()) // Volver al punto inicial

        if(boca==0){
            handlerFlechaCejas3.postDelayed({dibujarSeg2=true},2500)
            handlerFlechaCejas4.postDelayed({dibujarSeg1=true},5000)
        } else {
            dibujarSeg2=false
            dibujarSeg1=false
        }

        if(dibujarSeg2) {
            canvas.drawPath(pathSeg2, paintBlanco)
        } else {
            canvas.drawPath(pathSeg2, paintNegro)
        }
        if(dibujarSeg1){
            canvas.drawPath(pathSeg1, paintBlanco)
        } else {
            canvas.drawPath(pathSeg1, paintNegro)
        }
    }









    fun dibujaPuntosOjoDer(faceMeshpoints: MutableList<FaceMeshPoint>, canvas: Canvas, paintOjoDer: Paint, isTablet: Boolean, factorX: Float, factorY: Float, width: Int, height: Int){

        val puntosOjoDer = setOf(33, 7, 7, 163, 163, 144, 144, 145, 145, 153, 153, 154, 154, 155, 155, 133,
            33, 246, 246, 161, 161, 160, 160, 159, 159, 158, 158, 157, 157, 173, 173,
            133)

        for (i in puntosOjoDer){
            val position = faceMeshpoints[i].position
            if(UsageInfoFragment.visualizarDaltonismo){
                if(isTablet) { //Si es tablet
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) { //Si está en horizontal
                        canvas.drawRect((-position.x * factorX) + width - 5, (position.y * factorX) - (1188 - height) * factorX - 5, (-position.x * factorX) + width + 5, (position.y * factorX) - (1188 - height) * factorX + 5, paintOjoDer)
                    } else { //Si está en vertical
                        canvas.drawRect(((-position.x * 2.8) + width + 40 * factorX).toFloat() - 5, ((position.y * 2.8).toFloat()) + 10 * factorX - 5, (((-position.x * 2.8) + width + 40 * factorX).toFloat()) + 5, ((position.y * 2.8).toFloat()) + 10 * factorX + 5, paintOjoDer)
                    }
                } else { //Si es móvil
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
                        canvas.drawRect((-position.x * factorX) + width - 5, (position.y * factorX) - (1177 - height) * factorX - 5, (-position.x * factorX) + width + 5, (position.y * factorX) - (1177 - height) * factorX + 5, paintOjoDer)
                    } else { //Si está en vertical
                        canvas.drawRect((-position.x * 3.3f) + width + (80 * 3.2f) - 5, (position.y * 3.3f) + 30 - 5, (-position.x * 3.3f) + width + (80 * 3.2f) + 5, (position.y * 3.3f) + 30 + 5, paintOjoDer)
                    }
                }
            } else {
                if(isTablet) { //Si es tablet
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) { //Si está en horizontal
                        canvas.drawCircle((-position.x * factorX) + width, (position.y * factorX) - (1188 - height) * factorX, 5f, paintOjoDer)
                    } else { //Si está en vertical
                        canvas.drawCircle(((-position.x * 2.8) + width + 40 * factorX).toFloat(), ((position.y * 2.8).toFloat()) + 10 * factorX, 5f, paintOjoDer)
                    }
                } else { //Si es móvil
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
                        canvas.drawCircle((-position.x * factorX) + width, (position.y * factorX) - (1177 - height) * factorX, 5f, paintOjoDer)
                    } else { //Si está en vertical
                        canvas.drawCircle((-position.x * 3.3f) + width + (80 * 3.2f), (position.y * 3.3f) + 30, 5f, paintOjoDer)
                        //canvas.drawCircle((-position.x * factorY) + width + (80 * factorY), (position.y * factorY), 5f, paintOjoDer)
                    }
                }
            }
        }
//        val position = faceMeshpoints[101].position
//        val path = Path()
//        // Definir la forma de la flecha hacia abajo
//        path.moveTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1188 - height) * factorX + 50f) // Punto inicial
//        path.lineTo((-position.x * factorX) + width - 50f, (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea izquierda
//        path.lineTo((-position.x * factorX) + width - 25f, (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea inferior corta
//        path.lineTo((-position.x * factorX) + width - 25f, (position.y * factorX) - (1188 - height) * factorX - 50f) // Línea vertical
//        path.lineTo((-position.x * factorX) + width + 25f, (position.y * factorX) - (1188 - height) * factorX - 50f) // Línea inferior larga
//        path.lineTo((-position.x * factorX) + width + 25f, (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea derecha
//        path.lineTo((-position.x * factorX) + width + 50f, (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea superior corta
//        path.lineTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1188 - height) * factorX + 50f) // Volver al punto inicial
//
//        canvas.drawPath(path, paintOjoDer)
    }

    fun dibujaFlechaOjoDer(faceMeshpoints: MutableList<FaceMeshPoint>, canvas: Canvas, paintOjoDer: Paint, isTablet: Boolean, factorX: Float, factorY: Float, width: Int, height: Int, tamano: Double){
        val position = faceMeshpoints[101].position
        val path = Path()
        // Definir la forma de la flecha hacia abajo
        path.moveTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1188 - height) * factorX + (50f*(tamano*0.6).toFloat())) // Punto inicial
        path.lineTo((-position.x * factorX) + width - (50f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea izquierda
        path.lineTo((-position.x * factorX) + width - (25f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea inferior corta
        path.lineTo((-position.x * factorX) + width - (25f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX - (50f*(tamano*0.6).toFloat())) // Línea vertical
        path.lineTo((-position.x * factorX) + width + (25f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX - (50f*(tamano*0.6).toFloat())) // Línea inferior larga
        path.lineTo((-position.x * factorX) + width + (25f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea derecha
        path.lineTo((-position.x * factorX) + width + (50f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea superior corta
        path.lineTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1188 - height) * factorX + (50f*(tamano*0.6).toFloat())) // Volver al punto inicial


        val pathTabletVertical = Path()
        // Definir la forma de la flecha
        pathTabletVertical.moveTo((-position.x * 2.8f) + width + 40 * factorX + 0f, (position.y * 2.8f) + 10 * factorX + (50f*(tamano*0.6)).toFloat()) // Punto inicial
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX - (50f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (0f*(tamano*0.6)).toFloat()) // Línea izquierda
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX - (25f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (0f * (tamano*0.6)).toFloat()) // Línea inferior corta
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX - (25f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX - (50f * (tamano*0.6)).toFloat()) // Línea vertical
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX + (25f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX - (50f * (tamano*0.6)).toFloat()) // Línea inferior larga
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX + (25f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (0f * (tamano*0.6)).toFloat()) // Línea derecha
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX + (50f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (0f * (tamano*0.6)).toFloat()) // Línea superior corta
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX + 0f, (position.y * 2.8f) + 10 * factorX + (50f * (tamano*0.6)).toFloat()) // Volver al punto inicial


        val pathMovilVertical = Path()
        // Definir la forma de la flecha
        pathMovilVertical.moveTo((-position.x * 3.3f) + width + (80 * 3.2f) + 0f, (position.y * 3.3f) + 30 + (50f*(tamano*0.6)).toFloat()) // Punto inicial
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) - (50f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (0f*(tamano*0.6)).toFloat()) // Línea izquierda
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) - (25f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (0f * (tamano*0.6)).toFloat()) // Línea inferior corta
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) - (25f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 - (50f * (tamano*0.6)).toFloat()) // Línea vertical
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + (25f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 - (50f * (tamano*0.6)).toFloat()) // Línea inferior larga
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + (25f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (0f * (tamano*0.6)).toFloat()) // Línea derecha
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + (50f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (0f * (tamano*0.6)).toFloat()) // Línea superior corta
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + 0f, (position.y * 3.3f) + 30 + (50f * (tamano*0.6)).toFloat()) // Volver al punto inicial


        //canvas.drawPath(path, paintOjoDer)
        if(isTablet) { //Si es tablet
            if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) { //Si está en horizontal
                canvas.drawPath(path, paintOjoDer)
            } else { //Si está en vertical
                canvas.drawPath(pathTabletVertical, paintOjoDer)
            }
        } else { //Si es móvil
            if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
                canvas.drawPath(path, paintOjoDer)
            } else { //Si está en vertical
                canvas.drawPath(pathMovilVertical, paintOjoDer)
            }
        }
    }



    fun dibujaPuntosOjoIzq(faceMeshpoints: MutableList<FaceMeshPoint>, canvas: Canvas, paintOjoIzq: Paint, isTablet: Boolean, factorX: Float, factorY: Float, width: Int, height: Int){

        val puntosOjoIzq = setOf(263, 249, 249, 390, 390, 373, 373, 374, 374, 380, 380, 381, 381, 382, 382,
            362, 263, 466, 466, 388, 388, 387, 387, 386, 386, 385, 385, 384, 384, 398,
            398, 362)

        for (i in puntosOjoIzq){
            val position = faceMeshpoints[i].position
            if(UsageInfoFragment.visualizarDaltonismo){
                if(isTablet) { //Si es tablet
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) { //Si está en horizontal
                        canvas.drawRect((-position.x * factorX) + width - 5, (position.y * factorX) - (1188 - height) * factorX - 5, (-position.x * factorX) + width + 5, (position.y * factorX) - (1188 - height) * factorX + 5, paintOjoIzq)
                    } else { //Si está en vertical
                        canvas.drawRect(((-position.x * 2.8) + width + 40 * factorX).toFloat() - 5, ((position.y * 2.8).toFloat()) + 10 * factorX - 5, (((-position.x * 2.8) + width + 40 * factorX).toFloat()) + 5, ((position.y * 2.8).toFloat()) + 10 * factorX + 5, paintOjoIzq)
                    }
                } else { //Si es móvil
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
                        canvas.drawRect((-position.x * factorX) + width - 5, (position.y * factorX) - (1177 - height) * factorX - 5, (-position.x * factorX) + width + 5, (position.y * factorX) - (1177 - height) * factorX + 5, paintOjoIzq)
                    } else { //Si está en vertical
                        canvas.drawRect((-position.x * 3.3f) + width + (80 * 3.2f) - 5, (position.y * 3.3f) + 30 - 5, (-position.x * 3.3f) + width + (80 * 3.2f) + 5, (position.y * 3.3f) + 30 + 5, paintOjoIzq)
                    }
                }
            } else {
                if(isTablet) { //Si es tablet
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) { //Si está en horizontal
                        canvas.drawCircle((-position.x * factorX) + width, (position.y * factorX) - (1188 - height) * factorX, 5f, paintOjoIzq)
                    } else { //Si está en vertical
                        canvas.drawCircle(((-position.x * 2.8) + width + 40 * factorX).toFloat(), ((position.y * 2.8).toFloat()) + 10 * factorX, 5f, paintOjoIzq)
                    }
                } else { //Si es móvil
                    if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
                        canvas.drawCircle((-position.x * factorX) + width, (position.y * factorX) - (1177 - height) * factorX, 5f, paintOjoIzq)
                    } else { //Si está en vertical
                        canvas.drawCircle((-position.x * 3.3f) + width + (80 * 3.2f), (position.y * 3.3f) + 30, 5f, paintOjoIzq)
                        //canvas.drawCircle((-position.x * factorY) + width + (80 * factorY), (position.y * factorY), 5f, paintOjoIzq)
                    }
                }
            }
        }
//        val position = faceMeshpoints[330].position
//        val path = Path()
//        // Definir la forma de la flecha hacia abajo
//        path.moveTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1188 - height) * factorX + 50f) // Punto inicial
//        path.lineTo((-position.x * factorX) + width - 50f, (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea izquierda
//        path.lineTo((-position.x * factorX) + width - 25f, (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea inferior corta
//        path.lineTo((-position.x * factorX) + width - 25f, (position.y * factorX) - (1188 - height) * factorX - 50f) // Línea vertical
//        path.lineTo((-position.x * factorX) + width + 25f, (position.y * factorX) - (1188 - height) * factorX - 50f) // Línea inferior larga
//        path.lineTo((-position.x * factorX) + width + 25f, (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea derecha
//        path.lineTo((-position.x * factorX) + width + 50f, (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea superior corta
//        path.lineTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1188 - height) * factorX + 50f) // Volver al punto inicial
//
//        canvas.drawPath(path, paintOjoIzq)
    }

    fun dibujaFlechaOjoIzq(faceMeshpoints: MutableList<FaceMeshPoint>, canvas: Canvas, paintOjoIzq: Paint, isTablet: Boolean, factorX: Float, factorY: Float, width: Int, height: Int, tamano: Double){
        val position = faceMeshpoints[330].position
        val path = Path()
        // Definir la forma de la flecha hacia abajo
        path.moveTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1188 - height) * factorX + (50f*(tamano*0.6).toFloat())) // Punto inicial
        path.lineTo((-position.x * factorX) + width - (50f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea izquierda
        path.lineTo((-position.x * factorX) + width - (25f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea inferior corta
        path.lineTo((-position.x * factorX) + width - (25f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX - (50f*(tamano*0.6).toFloat())) // Línea vertical
        path.lineTo((-position.x * factorX) + width + (25f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX - (50f*(tamano*0.6).toFloat())) // Línea inferior larga
        path.lineTo((-position.x * factorX) + width + (25f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea derecha
        path.lineTo((-position.x * factorX) + width + (50f*(tamano*0.6).toFloat()), (position.y * factorX) - (1188 - height) * factorX + 0f) // Línea superior corta
        path.lineTo((-position.x * factorX) + width + 0f, (position.y * factorX) - (1188 - height) * factorX + (50f*(tamano*0.6).toFloat())) // Volver al punto inicial

        val pathTabletVertical = Path()
        // Definir la forma de la flecha
        pathTabletVertical.moveTo((-position.x * 2.8f) + width + 40 * factorX + 0f, (position.y * 2.8f) + 10 * factorX + (50f*(tamano*0.6)).toFloat()) // Punto inicial
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX - (50f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (0f*(tamano*0.6)).toFloat()) // Línea izquierda
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX - (25f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (0f * (tamano*0.6)).toFloat()) // Línea inferior corta
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX - (25f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX - (50f * (tamano*0.6)).toFloat()) // Línea vertical
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX + (25f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX - (50f * (tamano*0.6)).toFloat()) // Línea inferior larga
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX + (25f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (0f * (tamano*0.6)).toFloat()) // Línea derecha
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX + (50f * (tamano*0.6)).toFloat(), (position.y * 2.8f) + 10 * factorX + (0f * (tamano*0.6)).toFloat()) // Línea superior corta
        pathTabletVertical.lineTo((-position.x * 2.8f) + width + 40 * factorX + 0f, (position.y * 2.8f) + 10 * factorX + (50f * (tamano*0.6)).toFloat()) // Volver al punto inicial


        val pathMovilVertical = Path()
        // Definir la forma de la flecha
        pathMovilVertical.moveTo((-position.x * 3.3f) + width + (80 * 3.2f) + 0f, (position.y * 3.3f) + 30 + (50f*(tamano*0.6)).toFloat()) // Punto inicial
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) - (50f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (0f*(tamano*0.6)).toFloat()) // Línea izquierda
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) - (25f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (0f * (tamano*0.6)).toFloat()) // Línea inferior corta
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) - (25f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 - (50f * (tamano*0.6)).toFloat()) // Línea vertical
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + (25f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 - (50f * (tamano*0.6)).toFloat()) // Línea inferior larga
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + (25f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (0f * (tamano*0.6)).toFloat()) // Línea derecha
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + (50f * (tamano*0.6)).toFloat(), (position.y * 3.3f) + 30 + (0f * (tamano*0.6)).toFloat()) // Línea superior corta
        pathMovilVertical.lineTo((-position.x * 3.3f) + width + (80 * 3.2f) + 0f, (position.y * 3.3f) + 30 + (50f * (tamano*0.6)).toFloat()) // Volver al punto inicial


        //canvas.drawPath(path, paintOjoIzq)
        if(isTablet) { //Si es tablet
            if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) { //Si está en horizontal
                canvas.drawPath(path, paintOjoIzq)
            } else { //Si está en vertical
                canvas.drawPath(pathTabletVertical, paintOjoIzq)
            }
        } else { //Si es móvil
            if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
                canvas.drawPath(path, paintOjoIzq)
            } else { //Si está en vertical
                canvas.drawPath(pathMovilVertical, paintOjoIzq)
            }
        }
    }

}
