package com.example.domticacongestosfaciales

import android.Manifest
import android.content.Intent
import android.content.pm.PackageManager
import android.content.res.Configuration
import android.net.Uri
import android.os.Build
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.os.Environment
import android.provider.Settings
import android.util.Log
import android.view.MenuItem
import androidx.annotation.RequiresApi
import androidx.appcompat.app.ActionBarDrawerToggle
import androidx.appcompat.widget.Toolbar
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import androidx.core.view.GravityCompat
import androidx.drawerlayout.widget.DrawerLayout
import androidx.fragment.app.Fragment
import com.example.domticacongestosfaciales.sqlite.ConexionBBDDSQLite
import com.google.android.material.navigation.NavigationView

@RequiresApi(Build.VERSION_CODES.TIRAMISU)
class MainActivity : AppCompatActivity(), NavigationView.OnNavigationItemSelectedListener {

    //Variables para el uso del menú
    private lateinit var drawer: DrawerLayout
    private lateinit var toggle: ActionBarDrawerToggle

    //Para base de datos
    private var conexionBBDDSQLite: ConexionBBDDSQLite? = null

    //Creación de la actividad
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        //Llamada a la función permisos para solcitar permisos para bases de datos y Bluetooth
        permisos()

        //Asignación de la app bar de la interfaz con la variable toolbar
        val toolbar:Toolbar = findViewById(R.id.toolbar_main)
        setSupportActionBar(toolbar)

        // Cargar el fragment inicial debajo del Toolbar
        supportFragmentManager.beginTransaction()
            .replace(R.id.fragment_container, InitialFragment())
            .commit()

        //Se inicializa y configura la vista de fragmentos
        drawer = findViewById(R.id.drawer_layout)

        toggle = ActionBarDrawerToggle(this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close)
        drawer.addDrawerListener(toggle)

        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.setHomeButtonEnabled(true)

        //Se configura navigationView para manejar la navegación con el menú
        val navigationView: NavigationView = findViewById(R.id.nav_view)
        navigationView.setNavigationItemSelectedListener(this)


        //Inicialización de Base de datos
        conexionBBDDSQLite = ConexionBBDDSQLite(this)

        //Se crean las tablas de usuario y de topicos
        conexionBBDDSQLite!!.createTable_t_usuarios()
        conexionBBDDSQLite!!.createTable_t_topics()

        //Si la tabla está vacía (como al crearla), se introduce un usuario vacío
        if(conexionBBDDSQLite!!.check_t_usuarios()==0){
            val usuarios = llenarUsuarios()
            if(usuarios.isNotEmpty()){
                usuarios.forEach { usuario ->
                    conexionBBDDSQLite!!.insert_t_usuarios(usuario.dns, usuario.puerto, usuario.nombre, usuario.contrasena)
                }
            }
        }

        if(conexionBBDDSQLite!!.check_t_topics()==0){
            if(llenarTopics().size!=0){
                llenarTopics().forEach {
                    conexionBBDDSQLite!!.insert_t_topics(it)
                }
            }
        }
    }


//Base de datos
//He creado la "estructura" MeterUsuario para introducir las diferentes columnas de la tabla de usuarios
data class MeterUsuario(
    val dns: String,
    val puerto: Int,
    val nombre: String,
    val contrasena: String
)
    //Funciones de bases de datos para empezar con el usuario y un tópico ya en la DB
    fun llenarUsuarios(): ArrayList<MeterUsuario>{
        var arrayUsuarios: ArrayList<MeterUsuario> = ArrayList()
        arrayUsuarios.add(MeterUsuario("", 0, "", ""))
        return arrayUsuarios
    }

    fun llenarTopics(): ArrayList<String>{
        var arrayTopics: ArrayList<String> = ArrayList()
        arrayTopics.add("TopicPrueba")
        return arrayTopics
    }



    //Esta es la función que permitirá navegar entre fragmentos con el menú
    override fun onNavigationItemSelected(item: MenuItem): Boolean {
        when(item.itemId) {
            //Cada uno de las 5 opciones del menú lateral
            R.id.nav_item_one -> {
                // Cargar el fragment de la cámara
                supportFragmentManager.beginTransaction()
                    .replace(R.id.fragment_container, InitialFragment())
                    .addToBackStack(null)
                    .commit()
            }
            R.id.nav_item_two -> {
                // Cargar el fragment de configuración MQTT
                supportFragmentManager.beginTransaction()
                    .replace(R.id.fragment_container, MQTTConfigFragment())
                    .addToBackStack(null)
                    .commit()
            }
            R.id.nav_item_three -> {
                // Cargar el fragment de configuración Bluetooth
                supportFragmentManager.beginTransaction()
                    .replace(R.id.fragment_container, BTConfigFragment())
                    .addToBackStack(null)
                    .commit()
            }
            R.id.nav_item_four -> {
                // Cargar el fragment de ajustes
                supportFragmentManager.beginTransaction()
                    .replace(R.id.fragment_container, UsageInfoFragment())
                    .addToBackStack(null)
                    .commit()
            }

            R.id.nav_item_five -> {
                // Cargar el fragment de información sobre la APP
                supportFragmentManager.beginTransaction()
                    .replace(R.id.fragment_container, AppInfoFragment())
                    .addToBackStack(null)
                    .commit()
            }
        }
        drawer.closeDrawer(GravityCompat.START)
        return true
    }

    override fun onPostCreate(savedInstanceState: Bundle?) {
        super.onPostCreate(savedInstanceState)
        toggle.syncState()
    }

    override fun onConfigurationChanged(newConfig: Configuration) {
        super.onConfigurationChanged(newConfig)
        toggle.onConfigurationChanged(newConfig)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if(toggle.onOptionsItemSelected(item)){
            return true
        }
        return super.onOptionsItemSelected(item)
    }

    //He puesto aquí los permisos
    private fun permisos() {

        if (ContextCompat.checkSelfPermission(
                this,
                Manifest.permission.READ_SMS
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            ActivityCompat.requestPermissions(
                this, arrayOf(
                    Manifest.permission.BLUETOOTH,
                    Manifest.permission.BLUETOOTH_CONNECT,
                    Manifest.permission.BLUETOOTH_ADMIN,
                    Manifest.permission.BLUETOOTH_SCAN,
                    Manifest.permission.BLUETOOTH_ADVERTISE,
                    Manifest.permission.BLUETOOTH_PRIVILEGED,
                    Manifest.permission.MANAGE_DEVICE_POLICY_BLUETOOTH,
                    Manifest.permission.MANAGE_EXTERNAL_STORAGE,
                    Manifest.permission.READ_EXTERNAL_STORAGE,
                    Manifest.permission.WRITE_EXTERNAL_STORAGE
                    ),
                1
            )
        } else {
            Log.d("Bluetooth", "Todos los permisos concedidos ")
        }

        if (Build.VERSION.SDK_INT >= 30) {
            if (!Environment.isExternalStorageManager()) {
                try {
                    var uri = Uri.parse("package:" + packageName)

                    var intent = Intent(Settings.ACTION_MANAGE_APP_ALL_FILES_ACCESS_PERMISSION, uri)
                    startActivity(intent);
                } catch (ex: Exception) {
                    var intent = Intent();
                    intent.setAction(Settings.ACTION_MANAGE_ALL_FILES_ACCESS_PERMISSION)
                    startActivity(intent);
                }
            }
        }
    }
}
